-- Description : Provides an automatically generated button bar for clickable
-- quest items and for combinable "mote" type items.
--
-- Supports the /questclicks (and /qclicks) slash command
--
-- Valid slash commands are:
--
-- /qclicks - Opens the GUI panel
--
-- Additional Commands (Available, but better to use the GUI in most cases):
-- /qclicks enable - Enables the Add On
-- /qclicks disable - Disables the Add On
-- /qclicks help - Displays usage information
-- /qclicks status - Shows addon status information
-- /qclicks direction (up|down|left|right) - Sets the "grow" direction of the buttons
-- /qclicks resetpos - Resets the position of the anchor to the center of the screen
-- /qclicks update - Forces an update run of the button list (shouldn't be necessary)
-- /qclicks exclude (itemlink | itemname) - Excludes an item from the button bar
-- /qclicks unexclude (itemlink | itemname) - Removes an item from the exclusion list
-- /qclicks clearexcludes - Clears the exclusion list
-- /qclicks list - Shows the current exclusion list
-- /qclicks scale (1|2|3|4|5) - Sets the button scale
-- /qclicks lock - Toggles locking the bar from being dragged
-- /qclicks anchor - Toggles the visibility of the anchor frame
-- /qclicks addextra (itemlink) - adds an item that would not normally be listed to the bar
-- /qclicks removeextra (itemlink) - removes an item from the "extras" list
-- /qclicks extras - shows items in the "extras" list
-- /qclicks clearextra - clears the extras list
-- /qclicks collapse - toggles automatic buttons for Collapsible items on and off

-- Local variables that are not saved between sessions.
local addonName, addonTable = ...;

local QuestClicksTitleFrame                           -- Frame that holds the title bar and allows moving the buttons
local QuestClicksTextures = {}                        -- Textures for the buttons (derrived from the item icons)
local QuestClicksLinks = {}                           -- itemLinks for each button
local QuestClicksMacros = {}                          -- Macros for each button
local QuestClicksTypes = {}                           -- What type of buttons do we have
local QuestClicksStarted = false                      -- Set true when player enters world to prevent extra processing
local QuestClicksButtonSizes = {16, 24 ,32, 40, 64}   -- Possible button sizes
local QuestClicksSaveBackdrop                         -- Save the Anchor Frame backdrop for when it is hidden
local QuestClicksEmptyBackdrop                        -- Empty Backdrop Settings
local QuestClicksTitleLabel                           -- Label for the anchor frame
local QuestClicksCombinables = {}                     -- Crystalized Items to add
local QuestClicksMax = 12                             -- Max number of buttons to display on the bar
local QuestClicksLastButton = 0                       -- Last button clicked (for key binding)
local QuestClicksSubframes = {}                       -- Frames for the buttons themselves
local QCBindingKey=""                                 -- Key that the "Last Used" is bound to
local QCLazyUpdateCounter = 0
local QCNeedLazyUpdate = false
local QCDoneOnce = false
local QCSeenActivity = false

-- We now treat the player's inventory (worn items) as a bag.  We have a couple of new wrapper functions to support getting
-- slot information and item information from this "bag" so that we can eliminate two of the three loops in the
-- QuestClicksUpdateButtons function.
local QC_BagCount =  7
--
-- trial fix for 5.0 JLK
-- local QC_BagIDs = { -2, 0, 1, 2, 3, 4, 19 }         -- Internal bag numbers for Keyring (-2), Backpack (0), and bags (1-4)
-- local QC_ScanBags = { false, true, true, true, true, true, true }
local QC_BagIDs = { KEYRING_CONTAINER, -4, 0, 1, 2, 3, 4, 19 } 
local QC_ScanBags = { false, false, true, true, true, true, true, true }

-- Currently unused.  Part of an idea that may or may not work :)
local QC_BagStat = { true, true, true, true, true, true, true }

-- Pixel amounts to offset the Tool Tip Window to try to keep it from overlapping the bar.
-- Note that ToolTip modifying add-ons will often wreak havoc with this :)
local QuestClicksTooltipX = { ["down"] = 15, ["up"] = 15, ["left"] = 15, ["right"] = 15 }
local QuestClicksTooltipY = { ["down"] = -15, ["up"] = -15, ["left"] = -15, ["right"] = -15 }

-- Used to "cache" the contents of the bar.  A new bar will only be built if the
-- current bar and previous bar are different
local QCCurrentBar = {}
local QCPrevBar = {}

-- These variables track which Scrolling Combat Text add-ons are installed (if any)
local UseMSBT = false;
local UseSCT = false;
local UsePARROT = false;

-- Names of Inventory (Stuff you wear) slots.  API documentation recommends using these (global) names
-- instead of using the slot numbers.
local QuestClicksInventory = {
  "HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot", "WristSlot", "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot",
  "Finger0Slot", "Finger1Slot", "Trinket0Slot", "Trinket1Slot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot", "AmmoSlot"
}           

-- Used when examining recipies for skill requirements
local QuestClicksRecipeProfessions = {
  "Alchemy", "Blacksmithing", "Enchanting", "Engineering", "Inscription", "Jewelcrafting", "Leatherworking", 
  "Tailoring", "Archaeology", "Cooking", "First Aid"
}

-- Used to hold a list of the clauses we are searching for when checking items. Saves on some code instead of having them separate.
local QuestClicksClauseList = {}

-- QC_UseClauseCache and QC_QuestStarterCache hold the results of this session's Use Clause and Quest Starter
-- item scans so we only need to do the tooltip scanning once per item.
local QC_UseClauseCache = {}
local QC_ClauseCache = {}
local QC_QuestClauseCache = {}
local QC_ArtifactClauseCache = {}
local QC_ToyPetClauseCache = {}
local QC_GarrisonClauseCache = {}
local QC_EvalCache = {}
local QC_QuestStarterCache = {}
local QC_RecipeClauseCache = {}
local QC_FoodEvalResults = {}
local QC_DrinkEvalResults = {}
local QC_BlockersCache = {}

-- Whenever a "This Item Begins a Quest" item triggers an alert notification, we add it to QC_AlertedQuestStarters so that
-- we won't alert for it again during this session.
local QC_AlertedQuestStarters = {}

-- Get a local pointer to the "QuestClicks_GetLocalString" which lives in the Localization.lua file to support multiple languages
local L=QuestClicks_GetLocalizedString   

-- List of localized names of categories.  Each of these categories will get a check box in the config window to toggle adding that
-- type of item to the QuestClicks bar.  The contents of each category are specified in the QC_Conditionals list below, except for
-- the "Quest Starters" category, which is it's own special category with no actual content items.
local QC_CategoryNames = { L("Quest Starters"), L("Additions"), L("Combines"), L("Trades"), L("Keys"), L("Artifact Boosters"), L("Toys and Pets"), L("Garrison Rewards"), L("Recipes"), L("Lootable Containers"), L("Oneshot Boosts"), L("Engineer Tools") }

-- Items in the QC_Conditionals list will show up on the QuestClicks bar based on meeting certian criteria.  Each entry in the table
-- is in the following fomat:
--
-- ["ItemID"] = { Quantity, "RequiredSpell", "CategoryName" }
--
-- "ItemID" is a string representing the internal ID number of the item.
-- "Quantity" is the mininimum number of items present in inventory to trigger creating a button
-- "RequiredSpell" is the localized name of the spell that the character must know to allow a button to be built (nil=none required)
-- "CategoryName" is the localized name of the Category from QC_CategoryNames that governs filtering of this item in the config window
local QC_Conditionals = {
  -- ******************************************
  -- **  Force misc. quest items to show up  **
  -- ******************************************
  ["38657"] = {1, nil, L("Additions")},  -- Freya's Ward
  ["31121"] = {5, nil, L("Additions")},  -- Costume Scraps
  ["28038"] = {1, nil, L("Additions")},  -- Seaforium PU-36 Explosive Nether Modulator
  ["10515"] = {1, nil, L("Additions")},  -- Torch of Retribution
  ["35704"] = {1, nil, L("Additions")},  -- Incendiary Explosives
  ["34475"] = {1, nil, L("Additions")},  -- Arcane Charges
  ["32406"] = {1, nil, L("Additions")},  -- Skyguard Blasting Charges
  ["33634"] = {1, nil, L("Additions")},  -- Orehammer's Precision Bombs
  ["50218"] = {1, nil, L("Additions")},  -- Krennan's Potion of Stealth
  ["50220"] = {1, nil, L("Additions")},  -- Half-Burnt Torch
  ["51956"] = {1, nil, L("Additions")},  -- Blessed Offerings
  ["63351"] = {1, nil, L("Additions")},  -- Tahret Dynasty Mallet
  -- **************************
  -- **  Crystallized Items  **
  -- **************************
  ["37700"] = {10, nil, L("Combines")},  -- Crystallized Air
  ["37701"] = {10, nil, L("Combines")},  -- Crystallized Earth 
  ["37702"] = {10, nil, L("Combines")},  -- Crystallized Fire
  ["37703"] = {10, nil, L("Combines")},  -- Crystallized Shadow
  ["37704"] = {10, nil, L("Combines")},  -- Crystallized Life
  ["37705"] = {10, nil, L("Combines")},  -- Crystallized Water
  -- *************************
  -- **  Motes of Whatever  **
  -- *************************
  ["22572"] = {10, nil, L("Combines")},  -- Mote of Air
  ["22573"] = {10, nil, L("Combines")},  -- Mote of Earth
  ["22574"] = {10, nil, L("Combines")},  -- Mote of Fire
  ["22575"] = {10, nil, L("Combines")},  -- Mote of Life
  ["22576"] = {10, nil, L("Combines")},  -- Mote of Mana
  ["22577"] = {10, nil, L("Combines")},  -- Mote of Shadow
  ["22578"] = {10, nil, L("Combines")},  -- Mote of Water
  ["89112"] = {10, nil, L("Combines")},  -- Mote of Harmony
  -- **********************************
  -- **  Enchanting Lesser Essences  **
  -- **********************************
  ["34056"] = {3, nil, L("Combines")},   -- Lesser Cosmic Essence
  ["22447"] = {3, nil, L("Combines")},   -- Lesser Planar Essence
  ["16202"] = {3, nil, L("Combines")},   -- Lesser Eternal Essence
  ["11174"] = {3, nil, L("Combines")},   -- Lesser Nether Essence
  ["11134"] = {3, nil, L("Combines")},   -- Lesser Mystic Essence
  ["10998"] = {3, nil, L("Combines")},   -- Lesser Astral Essence
  ["10938"] = {3, nil, L("Combines")},   -- Lesser Magic Essence
  ["52718"] = {3, nil, L("Combines")},   -- Lesser Celestial Essence
  -- ****************************
  -- **    Skinning Scraps     **
  -- ****************************
  ["2934"]   = {3,  L("Skinning"), L("Combines")}, -- Ruined Leather Scraps
  ["25649"]  = {5,  L("Skinning"), L("Combines")}, -- Knothide Leather Scraps
  ["33567"]  = {5,  L("Skinning"), L("Combines")}, -- Borean Leather Scraps
  ["52977"]  = {5,  L("Skinning"), L("Combines")}, -- Savage Leather Scraps
  ["110610"] = {10, L("Skinning"), L("Combines")}, -- Raw Beast Hide Scraps
  -- ****************************
  -- **   Herbalism Partials   **
  -- ****************************
  ["109625"] = {10, nil, L("Combines")}, -- Broken Fireweed Stem
  ["109626"] = {10, nil, L("Combines")}, -- Gorgrond Flytrap Ichor
  ["109624"] = {10, nil, L("Combines")}, -- Broken Frostweed Stem
  ["109628"] = {10, nil, L("Combines")}, -- Nagrand Arrowbloom Petal
  ["109629"] = {10, nil, L("Combines")}, -- Talador Orchid Petal
  ["109627"] = {10, nil, L("Combines")}, -- Starflower Petal
  ["97619"]  = {10, nil, L("Combines")}, -- Torn Green Tea Leaf
  ["97620"]  = {10, nil, L("Combines")}, -- Rain Poppy Petal
  ["97621"]  = {10, nil, L("Combines")}, -- Silkweed Stem
  ["97622"]  = {10, nil, L("Combines")}, -- Snow Lily Petal
  ["97623"]  = {10, nil, L("Combines")}, -- Fool's Cap Spores
  ["97624"]  = {10, nil, L("Combines")}, -- Desecrated Herb Pod
  ["108359"] = {10, nil, L("Combines")}, -- Fireleaf Bramble
  ["108357"] = {10, nil, L("Combines")}, -- Talandra's Rose Petal
  ["108350"] = {10, nil, L("Combines")}, -- Nightmare Vine Stem
  ["108348"] = {10, nil, L("Combines")}, -- Ancient Lichen Petal
  ["108344"] = {10, nil, L("Combines")}, -- Felweed Stalk
  ["108345"] = {10, nil, L("Combines")}, -- Dreaming Glory Petal
  ["108347"] = {10, nil, L("Combines")}, -- Terocone Leaf
  ["108343"] = {10, nil, L("Combines")}, -- Icecap Petal
  ["108342"] = {10, nil, L("Combines")}, -- Sorrowmoss Leaf
  ["108341"] = {10, nil, L("Combines")}, -- Montain Silversage Stalk
  ["108339"] = {10, nil, L("Combines")}, -- Dreamfoil Blade
  ["108340"] = {10, nil, L("Combines")}, -- Golden Sansam Leaf
  ["108338"] = {10, nil, L("Combines")}, -- Gromsblood Leaf
  ["108337"] = {10, nil, L("Combines")}, -- Ghost Mushroom Cap
  ["108336"] = {10, nil, L("Combines")}, -- Blindweed Stem
  ["108335"] = {10, nil, L("Combines")}, -- Sungrass Stalk
  ["108332"] = {10, nil, L("Combines")}, -- Firebloom Petal
  ["108329"] = {10, nil, L("Combines")}, -- Dragon's Teeth Stem
  ["108326"] = {10, nil, L("Combines")}, -- Khadgar's Whisker Stem
  ["108331"] = {10, nil, L("Combines")}, -- Gondthorn Bramble
  ["108328"] = {10, nil, L("Combines")}, -- Fadeleaf Petal
  ["108325"] = {10, nil, L("Combines")}, -- Liferoot Stem
  ["108324"] = {10, nil, L("Combines")}, -- Kingsblood Petal
  ["108323"] = {10, nil, L("Combines")}, -- Wild Steelboom Petal
  ["108327"] = {10, nil, L("Combines")}, -- Grave Moss Leaf
  ["108330"] = {10, nil, L("Combines")}, -- Stranglekelp Blade
  ["108352"] = {10, nil, L("Combines")}, -- Goldclover Leaf
  ["108354"] = {10, nil, L("Combines")}, -- Tiger Lily Petal
  ["108355"] = {10, nil, L("Combines")}, -- Lichbloom Stalk
  ["108356"] = {10, nil, L("Combines")}, -- Icethorn Bramble
  ["108353"] = {10, nil, L("Combines")}, -- Adder's Tongue Stem
  ["108357"] = {10, nil, L("Combines")}, -- Talandra's Rose Petal
  ["108358"] = {10, nil, L("Combines")}, -- Deadnettle Bramble
  -- ****************************
  -- **  Nuggets and Fragments **
  -- ****************************
  ["108304"] = {10, nil, L("Combines")}, -- Khorium Ore Nugget
  ["108391"] = {10, nil, L("Combines")}, -- Titanium Ore Nugget
  ["108294"] = {10, nil, L("Combines")}, -- Silver Ore Nugget
  ["108302"] = {10, nil, L("Combines")}, -- Adamantite Ore Nugget
  ["108309"] = {10, nil, L("Combines")}, -- Pyrite Ore Nugget
  ["109991"] = {10, nil, L("Combines")}, -- True Iron Ore Nugget
  ["97512"]  = {10, nil, L("Combines")}, -- Ghost Iron Nugget
  ["97546"]  = {10, nil, L("Combines")}, -- Kyparite Fragment
  ["108301"] = {10, nil, L("Combines")}, -- Fel Iron Ore Nugget
  ["108300"] = {10, nil, L("Combines")}, -- Mithril Ore Nugget
  ["108298"] = {10, nil, L("Combines")}, -- Thorium Ore Nugget
  ["108297"] = {10, nil, L("Combines")}, -- Iron Ore Nugget
  ["132989"] = {10, nil, L("Combines")}, -- Legion Iron Nugget
  ["108305"] = {10, nil, L("Combines")}, -- Cobalt Ore Nugget
  ["108306"] = {10, nil, L("Combines")}, -- Saronite Ore Nugget
  ["108303"] = {10, nil, L("Combines")}, -- Eternium Ore Nugget
  ["108296"] = {10, nil, L("Combines")}, -- Gold Ore Nugget
  ["108299"] = {10, nil, L("Combines")}, -- Truesilver Ore Nugget
  ["108295"] = {10, nil, L("Combines")}, -- Tin Ore Nugget
  ["108307"] = {10, nil, L("Combines")}, -- Obsidium Ore Nugget
  ["108308"] = {10, nil, L("Combines")}, -- Elementium Ore Nugget
  ["109992"] = {10, nil, L("Combines")}, -- Blackrock Fragment

  -- ****************************
  -- **  Ores for prospecting  **
  -- ****************************
  ["2770"] = {5, L("Prospecting"), L("Trades")},  -- Copper Ore
  ["2771"] = {5, L("Prospecting"), L("Trades")},  -- Tin Ore
  ["2772"] = {5, L("Prospecting"), L("Trades")},  -- Iron Ore
  ["3858"] = {5, L("Prospecting"), L("Trades")},  -- Mithril Ore
  ["10620"] = {5, L("Prospecting"), L("Trades")}, -- Thorium Ore
  ["23424"] = {5, L("Prospecting"), L("Trades")}, -- Fel Iron Ore
  ["23425"] = {5, L("Prospecting"), L("Trades")}, -- Adamantite Ore
  ["36909"] = {5, L("Prospecting"), L("Trades")}, -- Cobalt Ore
  ["36912"] = {5, L("Prospecting"), L("Trades")}, -- Saronite Ore
  ["36910"] = {5, L("Prospecting"), L("Trades")}, -- Titanium Ore
  ["53038"] = {5, L("Prospecting"), L("Trades")}, -- Obsidium Ore
  ["52185"] = {5, L("Prospecting"), L("Trades")}, -- Elementium Ore
  ["52183"] = {5, L("Prospecting"), L("Trades")}, -- Pyrite Ore
  ["72092"] = {5, L("Prospecting"), L("Trades")}, -- Ghost Iron Ore
  ["72093"] = {5, L("Prospecting"), L("Trades")}, -- Kyparite Ore
  ["72094"] = {5, L("Prospecting"), L("Trades")}, -- Black Trillium Ore
  ["72103"] = {5, L("Prospecting"), L("Trades")}, -- White Trillium Ore
  ["123918"] = {5, L("Prospecting"), L("Trades")}, -- Leystone Ore
  -- ************************
  -- **  Tailorling Cloth  **
  -- ************************
  ["2589"] = {2, L("Tailoring"), L("Trades")},      -- Linen Cloth
  ["2592"] = {3, L("Tailoring"), L("Trades")},      -- Wool Cloth
  ["4306"] = {4, L("Tailoring"), L("Trades")},      -- Silk Cloth
  ["4338"] = {4, L("Tailoring"), L("Trades")},      -- Mageweave Cloth
  ["14047"] = {4, L("Tailoring"), L("Trades")},     -- Runecloth
  ["21877"] = {5, L("Tailoring"), L("Trades")},     -- Netherweave Cloth
  ["33470"] = {5, L("Tailoring"), L("Trades")},     -- Frostweave Cloth
  ["53010"] = {5, L("Tailoring"), L("Trades")},     -- Embersilk Cloth
  -- ["54440"] = {5, L("Tailoring"), L("Trades")},  -- Dream Cloth is elemental Embersilk
  ["72988"] = {5, L("Tailoring"), L("Trades")},     -- Windwool Cloth
  -- *************************
  -- **  Herbs for Milling  **
  -- *************************
  ["765"] = {5, L("Milling"), L("Trades")},     -- Silverleaf
  ["2447"] = {5, L("Milling"), L("Trades")},    -- Peacebloom
  --["22710"] = {5, L("Milling"), L("Trades")}, -- Bloodthistle is not Millable, so disable
  ["2449"] = {5, L("Milling"), L("Trades")},    -- Earthroot
  ["785"] = {5, L("Milling"), L("Trades")},     -- Mageroyal
  ["2452"] = {5, L("Milling"), L("Trades")},    -- Swiftthistle
  ["2450"] = {5, L("Milling"), L("Trades")},    -- Briarthorn
  ["3820"] = {5, L("Milling"), L("Trades")},    -- Stranglekelp
  ["2453"] = {5, L("Milling"), L("Trades")},    -- Bruiseweed
  ["3355"] = {5, L("Milling"), L("Trades")},    -- Wild Steelbloom
  ["3369"] = {5, L("Milling"), L("Trades")},    -- Grave Moss
  ["3356"] = {5, L("Milling"), L("Trades")},    -- Kingsblood
  ["3357"] = {5, L("Milling"), L("Trades")},    -- Liferoot
  ["3818"] = {5, L("Milling"), L("Trades")},    -- Fadeleaf
  ["3821"] = {5, L("Milling"), L("Trades")},    -- Goldthorn
  ["3358"] = {5, L("Milling"), L("Trades")},    -- Khadgar's Whisker
  ["3819"] = {5, L("Milling"), L("Trades")},    -- Dragon's Teeth
  ["4625"] = {5, L("Milling"), L("Trades")},    -- Firebloom
  ["8831"] = {5, L("Milling"), L("Trades")},    -- Purple Lotus 
  ["8836"] = {5, L("Milling"), L("Trades")},    -- Arthas' Tears
  ["8838"] = {5, L("Milling"), L("Trades")},    -- Sungrass
  ["8839"] = {5, L("Milling"), L("Trades")},    -- Blindweed
  ["8845"] = {5, L("Milling"), L("Trades")},    -- Ghost Mushroom
  ["8846"] = {5, L("Milling"), L("Trades")},    -- Gromsblood
  ["13464"] = {5, L("Milling"), L("Trades")},   -- Golden Sansam
  ["13463"] = {5, L("Milling"), L("Trades")},   -- Dreamfoil
  ["13465"] = {5, L("Milling"), L("Trades")},   -- Mountain Silversage
  ["13466"] = {5, L("Milling"), L("Trades")},   -- Sorrowmoss
  ["13467"] = {5, L("Milling"), L("Trades")},   -- Icecap
  --["13468"] = {5, L("Milling"), L("Trades")}, -- Black Lotus is not Millable, so disable
  ["22785"] = {5, L("Milling"), L("Trades")},   -- Felweed
  ["39970"] = {5, L("Milling"), L("Trades")},   -- Fire Leaf
  ["22786"] = {5, L("Milling"), L("Trades")},   -- Dreaming Glory
  --["22788"] = {5, L("Milling"), L("Trades")}, -- Flame Cap is not Millable, so disable
  ["22789"] = {5, L("Milling"), L("Trades")},   -- Terocone
  ["22790"] = {5, L("Milling"), L("Trades")},   -- Ancient Lichen
  ["22791"] = {5, L("Milling"), L("Trades")},   -- Netherbloom
  ["22792"] = {5, L("Milling"), L("Trades")},   -- Nightmare Vine
  ["22793"] = {5, L("Milling"), L("Trades")},   -- Mana Thistle
  ["22794"] = {5, L("Milling"), L("Trades")},   -- Fel Lotus
  ["36901"] = {5, L("Milling"), L("Trades")},   -- Goldclover
  ["36907"] = {5, L("Milling"), L("Trades")},   -- Talandra's Rose
  ["36904"] = {5, L("Milling"), L("Trades")},   -- Tiger Lily
  ["36903"] = {5, L("Milling"), L("Trades")},   -- Adder's Tongue
  ["37921"] = {5, L("Milling"), L("Trades")},   -- Deadnettle
  ["36905"] = {5, L("Milling"), L("Trades")},   -- Lichbloom
  ["36906"] = {5, L("Milling"), L("Trades")},   -- Icethorn
  --["36908"] = {5, L("Milling"), L("Trades")}, -- Frost Lotus is not Millable, so disabled
  ["22787"] = {5, L("Milling"), L("Trades")},   -- Ragveil
  ["52983"] = {5, L("Milling"), L("Trades")},   -- Cinderbloom
  ["52984"] = {5, L("Milling"), L("Trades")},   -- Stormvine
  ["52985"] = {5, L("Milling"), L("Trades")},   -- Azshara's Veil
  ["52986"] = {5, L("Milling"), L("Trades")},   -- Heartblossom
  ["52987"] = {5, L("Milling"), L("Trades")},   -- Twilight Jasmine
  ["52988"] = {5, L("Milling"), L("Trades")},   -- Whiptail
  ["72237"] = {5, L("Milling"), L("Trades")},   -- Rain Poppy
  ["72234"] = {5, L("Milling"), L("Trades")},   -- Green Tea Leaf
  ["52989"] = {5, L("Milling"), L("Trades")},   -- Death Spore
  ["72235"] = {5, L("Milling"), L("Trades")},   -- Silk Weed
  ["79010"] = {5, L("Milling"), L("Trades")},   -- Snow Lily
  ["79011"] = {5, L("Milling"), L("Trades")},   -- Fool's Cap
  --["72238"] = {5, L("Milling"), L("Trades")}, -- Golden Lotus is not Millable, so disabled
}

-- Local references to the TK config interface library functions
local TKheading, TKcheckbox, TKbutton, TKslider

-- QC_ConfigCheckboxes holds the objects associated with the configuration checkboxes create for the QC_CategoryNames categories.
local QC_ConfigCheckboxes = {}

-- QC_ConfigChecksX and QC_ConfigChecksY hold the location of the first checkbox created for QC_CategoryNames categories.
local QC_ConfigChecksX = 0
local QC_ConfigChecksY = -180

-- Each subsequent checkbox is offset by QC_ConfigChecksVerticalSpacing pixels.
local QC_ConfigChecksVerticalSpacing = -20

-- Check Boxes for non-category configuration options
local cfg_chkEnabled, cfg_chkAnchor, cfg_chkLocked, cfg_chkKeybindEnabled
local cfg_lblDirection, cfg_slideScale, cfg_lblScale, cfg_lblKeybindEnabled, cfg_slidePerLine, cfg_lblPerLine
local cfg_lblKeybindKey, cfg_btnKeybindKey

-- Determine if the configuration page has been opened yet during this session
local ConfigPageLoaded = false

-- This is the actual Blizzard Configuration Panel frame that we will add controls to and insert into the Interface/Addons menu
local QCOptionsPanel = CreateFrame("Frame", nil, UIParent)

-- Support for ButtonFacade if it exists
local LBF, minor = LibStub:GetLibrary("LibButtonFacade", true)
if minor then
  if minor < 40000 then LBF=nil end
end

local QCBF, QCBF_Group

-- QuestClicksProfile is stored between sessions and contains the configuration data for the addon.
-- These are the default values and will be used if the addon hasn't been run before.
QuestClicksProfile = {
  ["Enabled"] = true,               -- If "true", QuestClicks is enabled
  ["Direction"] = "down",           -- Direction the button bar will "Grow"
  ["Anchor"] = "CENTER",            -- UI Point that the anchor frame is anchored to.
  ["RelativePoint"] = "CENTER",     -- Point withing the anchor that the position references
  ["X"] = 0,                        -- X offset from the "Anchor" point
  ["Y"] = 0,                        -- Y offset from teh "Anchor" point
  ["Excludes"]  = {},               -- Items in the Excludes list will never get buttons
  ["Includes"] = {},                -- Items in the Includes list will get buttons even if they don't meet normal criteria
  ["ButtonSize"] = 3,               -- Index into QuestClicksButtonsSizes to determine size of each button
  ["Locked"] = false,               -- If "true", the QuestClicks bar cannot be moved
  ["AnchorHidden"] = false,         -- If "true", the QuestClicks Anchor Frame (that says "Clicks") will be hidden
  ["Collapsibles"] = true,          -- Variable no longer used
  ["Categories"] = {                -- Flags for the categories in QC_CategoryNames.  If true, they will be active
    [L("Additions")] = true,
	[L("Combines")] = true,
	[L("Trades")] = true,
	[L("Keys")] = false,
    [L("Quest Starters")] = true,
	[L("Artifact Boosters")] = true,
	[L("Toys and Pets")] = true,
	[L("Garrison Rewards")] = true,
	[L("Recipes")] = true,
	[L("Lootable Containers")] = true,
	[L("Oneshot Boosts")] = true,
	[L("Engineer Tools")] = true,
	[L("Best Food Item")] = true,
	[L("Best Drink Item")] = true
  },
  ["ButtonFacade"] = {              -- ButtonFacade skinning information
    ["Skin"] = "Blizzard",
	["Gloss"] = 0,
	["Backdrop"] = false
  },
  ["Padding"] = 0,                  -- Space (pixels) between each button
  ["DebugMode"] = false,            -- Don't turn this on unless you really like spam
  ["QuestStarterNotify"] = true,    -- If "true", Display alert notifications when you loot a Quest Starter item.
  ["EnableKeybinding"] = false,     -- If "true", every time a button is clicked the bind key will be set to that button's item
  ["Bindkey"] = "CTRL-Q",
  ["ButtonsPerLine"] = 12           -- Number of buttons to display on a single row before wrapping
}

---
---
---  Initialization and Setup
---
---
--- QuestClicksLoad is called when the addon is loaded.  It is important to note that we haven't yet
--- loaded any saved variables, and probably don't have access to much in-game information yet, so we will
--- simply build some of the things we need here and subscribe to events.
function QuestClicks_OnLoad(self)
    
	-- QuestClicks_DisplayMessage("Loaded");
	
	self:SetScript("OnEvent", QuestClicks_OnEvent);
	
    -- Register the VARIABLES_LOADED event so we can build our initial list of buttons
    self:RegisterEvent("VARIABLES_LOADED");
	
    -- Register this so we can not process the buttons until the player enters the game... this
    -- just saves some loading time.
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	self:RegisterEvent("PLAYER_LEAVING_WORLD");

    -- The BAG_UPDATE event is called whenever the player's inventory changes
    self:RegisterEvent("BAG_UPDATE");
	
	-- The BANKFRAME_CLOSED event triggers a full button update.
	self:RegisterEvent("BANKFRAME_CLOSED");
    
    -- We'll also update after the player leaves combat, since we can't update in combat
    self:RegisterEvent("PLAYER_REGEN_ENABLED");
    
    -- Trap the Cooldown to update our own cooldown frame
    self:RegisterEvent("BAG_UPDATE_COOLDOWN");

    -- Trap the Cooldown for inventory (worn) items
    self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

    self:RegisterEvent("CHAT_MSG_SKILL");
	
	self:RegisterEvent("PLAYER_LEVEL_UP");
	
	self:RegisterEvent("QUEST_FINISHED");
	
	self:RegisterEvent("TOYS_UPDATED");
	
	self:RegisterEvent("COMPANION_LEARNED");
	
    -- Update the Binding Key when bindings change
    --self:RegisterEvent("UPDATE_BINDINGS");
    
    -- Create a "label" frame which we will use to drag the button bar around
    QuestClicksTitleFrame=CreateFrame("FRAME","QuestClicksTitleFrame",UIParent);
    QuestClicksTitleFrame:SetWidth(46); 
    QuestClicksTitleFrame:SetHeight(20);
    QuestClicksTitleFrame:SetPoint(QuestClicksProfile["RelativePoint"], UIParent, QuestClicksProfile["Anchor"], QuestClicksProfile["X"], QuestClicksProfile["Y"]);
    -- Save the "Backdrop" of the "Clicks" window while it is empty so we can use it later when we want to hide the Anchor Frame
    QuestClicksEmptyBackdrop = QuestClicksTitleFrame:GetBackdrop()
    QuestClicksTitleFrame:SetBackdrop({
      bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
      edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", 
      tile=1, tileSize=10, edgeSize=10, 
      insets={left=3, right=3, top=3, bottom=3}
    });
    
    -- Save the "Filled" backdrop of the "Clicks" window so we can easily put it back when "unhiding" the Anchor Frame
    QuestClicksSaveBackdrop = QuestClicksTitleFrame:GetBackdrop()
    
    -- Make the frame movable via the left mouse button
    QuestClicksTitleFrame:SetMovable(true)
    QuestClicksTitleFrame:EnableMouse(true)
    QuestClicksTitleFrame:RegisterForDrag("LeftButton")
    
    -- By setting these scripts, we actually let the frame be dragged around.,.. otherwise
    -- dragging it wouldn't do anything.
    QuestClicksTitleFrame:SetScript("OnDragStart", function(this) 
      if QuestClicksProfile.Locked  == false then
          if QuestClicksProfile.AnchorHidden == false then
            this:StartMoving() 
            end
        end
    end)
    
    -- When dragging is finished, get the location of the frame and store it to our variables.
    -- Interestingly, when a frame is moved it's anchor dynamically updates to the closest
    -- anchor point (the relativePoint value), so we have to store not only the X and Y offsets
    -- but what point they are relative too as well.
    QuestClicksTitleFrame:SetScript("OnDragStop", function(this) 
      this:StopMovingOrSizing() 
      local point, relativeTo, relativePoint, xOfs, yOfs = QuestClicksTitleFrame:GetPoint()
      QuestClicksProfile["X"] = xOfs;
      QuestClicksProfile["Y"] = yOfs;
      QuestClicksProfile["Anchor"] = relativePoint
	  QuestClicksProfile["RelativePoint"] = relativeTo
    end)

    -- Handle Click events on the Anchor Frame.  You can right click to expand/collapse, or
    -- left click with an item in hand to add it to the Extras List
    QuestClicksTitleFrame:SetScript("OnMouseUp", function(this,button)
      -- When the user right-clicks on the Anchor frame, all of the buttons will be hidden and the
      -- "Clicks" text will turn red.  Right-clicking again re-expands the button bar.
      -- This is actually a trick of simply setting the "Max" buttons to 0 to prevent them from being
      -- generated/displayed.    
      if button == "RightButton" then
        if QuestClicksMax == 12 then
          QuestClicksTitleLabel:SetTextColor(1,0,0)
          QuestClicksMax = 0
        else
          QuestClicksTitleLabel:SetTextColor(1,0.9,0)
          QuestClicksMax = 12
        end
        QuestClicksUpdateButtons();
      end
      
      -- Left clicking on the Anchor Frame with an item on your cursor will add that item to the
      -- "Extras" list.
      if button == "LeftButton" then
        local infotype, itemid, itemlink = GetCursorInfo()
        if infotype == "item" then
          QuestClicks_SlashHandler("addextra " .. itemlink)
          ClearCursor()
        end
      end
    end)
      
    -- create a string to put into the title bar frame
    QuestClicksTitleLabel = QuestClicksTitleFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormal");
    QuestClicksTitleLabel:SetAllPoints();
    QuestClicksTitleLabel:SetText("Clicks");
    QuestClicksTitleLabel:SetTextColor(1,0.9,0)
        
    -- Create frames for each of the buttons.  They will start out empty and hidden.  We pre-create all of our frames
    -- to save time at run-time by not creating and destroying a bunch of frames then.
    for i=1, QuestClicksMax do
      QuestClicksSubframes[i] = CreateFrame("Button", "QuestClicksSubrame" .. i, QuestClicksTitleFrame, "SecureActionButtonTemplate")
      QuestClicksSubframes[i]:SetWidth(QuestClicksButtonSizes[QuestClicksProfile.ButtonSize])
      QuestClicksSubframes[i]:SetHeight(QuestClicksButtonSizes[QuestClicksProfile.ButtonSize])
      QuestClicksSubframes[i]:SetPoint("TOP", QuestClicksTitleFrame, "BOTTOM", 0, -1 * ((i-1) * 32))
        
      QuestClicksSubframes[i].ItemCount = QuestClicksSubframes[i]:CreateFontString("$parentText", "ARTWORK", "GameFontNormal")
      QuestClicksSubframes[i].ItemCount:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
      QuestClicksSubframes[i].ItemCount:SetTextColor(1,0.9,0)
      QuestClicksSubframes[i].ItemCount:SetPoint("BOTTOMRIGHT", QuestClicksSubframes[i], "BOTTOMRIGHT", -3, 3)
      QuestClicksSubframes[i].ItemCount:SetText("")
        
      -- Create a coolown overlay for the button
      QuestClicksSubframes[i].Cooldown = CreateFrame("Cooldown", nil, QuestClicksSubframes[i], "CooldownFrameTemplate")
      QuestClicksSubframes[i].Cooldown:SetAllPoints()
      
      -- Allow this button to respond to any type of mouse button "Up" event      
      QuestClicksSubframes[i]:RegisterForClicks("AnyUp")
        
      -- Create textures for the buttons.  They start out blank, but will be sized
      -- to the size of the underlying button.
      --QuestClicksTextures[i] = QuestClicksSubframes[i]:CreateTexture(nil,"BACKGROUND")
      --QuestClicksTextures[i]:SetAllPoints(QuestClicksSubframes[i])
	  QuestClicksSubframes[i].Icon = QuestClicksSubframes[i]:CreateTexture(nil,"ARTWORK")
	  QuestClicksSubframes[i].Icon:SetAllPoints(QuestClicksSubframes[i])
        
      -- Set scripts to show/hide the tooltip when the player mouses over the button.  We have to set the
      -- owner of the tooltip each time, because Hide() clears it and it will jump all over if we don't
      QuestClicksSubframes[i]:SetScript("OnEnter", function(this)
        if QuestClicksSubframes[i]:IsVisible() then
          if QuestClicksLinks[i] then
            GameTooltip:SetOwner(QuestClicksSubframes[i], "ANCHOR_BOTTOMRIGHT", QuestClicksTooltipX[QuestClicksProfile.Direction], QuestClicksTooltipY[QuestClicksProfile.Direction])
            GameTooltip:SetHyperlink(QuestClicksLinks[i])
            --QuestClicks_DisplayMessage(QuestClicksSubframes[i]:GetAttribute("type*"))
            if QuestClicksSubframes[i]:GetAttribute("type*") == "macro" then
              GameTooltip:AddLine("QuestClicks: Click to Mill/Prospect/Combine this item", 1,0,0)
            end
            GameTooltip:Show()
          end
        end
      end)
        
      QuestClicksSubframes[i]:SetScript("OnMouseUp", function(this)
        if (QuestClicksLastButton > 0) and (QuestClicksProfile.EnableKeybinding == true) and (QuestClicksProfile.Bindkey ~= "") then
          --if not (QuestClicksLastButton == i) then
            if QuestClicksTypes[i] == "M" then
              SetBindingMacro(QuestClicksProfile.Bindkey, QuestClicksMacros[i]);
            else
              SetBindingItem(QuestClicksProfile.Bindkey, QuestClicksLinks[i]);
            end
            QuestClicksLastButton = i;
          --end
        end        
      end)
        
      -- Sets the script to hide the Tooltip when the mouse leaves the button
      QuestClicksSubframes[i]:SetScript("OnLeave", function(this)
        GameTooltip:Hide()
      end)
	  
      -- Create a "Scanning Tooltip" that we can use later to check for "Use:" and "This Item Begins a Quest" on items  
      CreateFrame( "GameTooltip", "QuestClicksScanningTooltip", nil, "GameToolTipTemplate" ); -- Tooltip name cannot be nil
      QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
      
    end

    -- Set up our Slash Command and Slash Command Handler function
    SLASH_QuestClicks1 = "/questqlicks";
    SLASH_QuestClicks2 = "/qclicks"; -- A shortcut or alias
    SlashCmdList["QuestClicks"] = QuestClicks_SlashHandler;
	
	-- Set up the Options panel
	QCOptionsPanel.name = "QuestClicks"
	QCOptionsPanel:Hide()
    -- When the panel is shown, we want to call "QuestClicksOptionsSetup" to make sure the controls exist and have the right values in them
	QCOptionsPanel:SetScript("OnShow", function() QuestClicksOptionsSetup() end)
    -- When the panel is hidden, we want to update the button bar, just to make sure we have everything right.
	QCOptionsPanel:SetScript("OnHide", function() QuestClicksUpdateButtons() end)
    -- InterfaceOptions_AddCategory actually adds our panel to the Interface/Addons menu in the Blizzard config system.
	InterfaceOptions_AddCategory(QCOptionsPanel)
	QuestClicksUpdateClauseList()
end

-- Simple message broadcaster for our Quest Starter alerts.  Will add a message to the UIErrorsFrame,
-- the chat window, and to your Scrolling Combat Text addon (if available)
local function QCMessage(msg)

  DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 1.0, 0)

  UIErrorsFrame:AddMessage(msg, 1.0, 1.0, 0, 53, 45);

  -- If enabled, display a message using MikScrollingBattleText
  if UseMSBT then
    MikSBT.DisplayMessage(msg,MikSBT.DISPLAYTYPE_NOTIFICATION, true, 255, 0, 255, nil, nil, nil, icon)
    return
  end
				
  -- If enabled, display a message using SCT (Scrolling Combat Text)
  if UseSCT then
    local rgbcolor = { r=1, g=0, b=1 };
    SCT:DisplayMessage(msg, rgbcolor);
    return
  end
				
  -- If enabled, display a message using Parrot
  if UsePARROT then
    Parrot:ShowMessage(msg, "Notification", true, 1, 0, 1, nil, nil, "NORMAL", icon);
    return
  end
end

-- Debug messages call this function, which does nothing if the user is not in debug mode.  Otherwise,
-- the message is passed along to QCMessage
local function QCDebugMessage(msg)
  if QuestClicksProfile.DebugMode == true then
    QCMessage(msg)
  end
end
---
---
---  Configuration GUI Stuff
---
---

-- Create a label control on the Options panel and set it's text to "subtext"
function QuestClicksOptionsLabel(parent, subtext, x, y, h, w)
    -- Note, suggested height is 32
	local subtitle = parent:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	subtitle:SetHeight(h)
	subtitle:SetWidth(w)
	subtitle:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	subtitle:SetNonSpaceWrap(true)
	subtitle:SetJustifyH("LEFT")
	subtitle:SetJustifyV("TOP")
	subtitle:SetText(subtext)
	return subtitle
end

-- Configure the options panel the first time it is shown.
-- Note: Some of these objects are defined here as "local".  This is mostly true for labels that don't need to change.
-- Other options are defined in the locals at the top of this file, because other functions in the file need to be able
-- to update them when configuration options change via slash commands.
function QuestClicksOptionsSetup()

  -- If the options panel has already been loaded, we don't need to do all of this again.  If it hasn't, we are going
  -- to create all of our options panel controls.
  if ConfigPageLoaded == false then
  
    -- Set the ConfigPageLoaded value to true so we don't run this more than once.
    ConfigPageLoaded = true
    
    -- Create a "TKheading" object, which is the larger title at the top of the frame.
    local title = TKheading(QCOptionsPanel, L("QuestClicks Configuration Options"))

    ---
    --- General check box options
    ---
    
    -- Create check boxes for the various options.  
    
    -- QuestClicksProfile.Enabled option Check Box
	cfg_chkEnabled = TKcheckbox(QCOptionsPanel, 26, L("Enable QuestClicks"), "TOPLEFT", title, "TOPLEFT", 0, -20)
	cfg_chkEnabled.tiptext = L("Enable or Disable the QuestClicks addon")
    cfg_chkEnabled:SetChecked(QuestClicksProfile["Enabled"])
	local orig=cfg_chkEnabled:GetScript("OnClick")
	cfg_chkEnabled:SetScript("OnClick", function(self)
	  orig(self)
      if self:GetChecked() then
        QuestClicksProfile["Enabled"] = true;
        QuestClicks_SlashHandler("enable")
      else
        QuestClicksProfile["Enabled"] = false;
        QuestClicks_SlashHandler("disable")
      end
	  QuestClicksProfile["Enabled"] = self:GetChecked()
    end
	)
	
    -- QuestClicksProfile.Locked option Check Box
    cfg_chkLock = TKcheckbox(QCOptionsPanel, 26, L("Lock Position"), "TOPLEFT", title, "TOPLEFT", 0, -40)
	cfg_chkLock.tiptext = L("When locked, the QuestClicks bar cannot be moved.")
	cfg_chkLock:SetChecked(QuestClicksProfile.Locked)
	local orig=cfg_chkLock:GetScript("OnClick")
	cfg_chkLock:SetScript("OnClick", function(self)
	  orig(self)
	  if self:GetChecked() then
	    QuestClicksProfile.Locked = true
	  else
	    QuestClicksProfile.Locked = false
	  end
    end
	)

    -- QuestClicksProfile.AnchorHidden option Check Box
	cfg_chkAnchor = TKcheckbox(QCOptionsPanel, 26, L("Hide Anchor Frame"), "TOPLEFT", title, "TOPLEFT", 0, -60)
	cfg_chkAnchor.tiptext = L("When checked, the title/anchor frame (displays 'Clicks') is hidden.")
	cfg_chkAnchor:SetChecked(QuestClicksProfile.AnchorHidden)
	local orig=cfg_chkAnchor:GetScript("OnClick")
	cfg_chkAnchor:SetScript("OnClick", function(self)
	  orig(self)
	  if self:GetChecked() then
	    QuestClicksProfile.AnchorHidden = true
	  else
	    QuestClicksProfile.AnchorHidden = false
	  end
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
	)
    
    -- QuestClicksProfile.QuestStarterNotify options Check Box
    cfg_chkNotify = TKcheckbox(QCOptionsPanel, 26, L("Quest Starter Alert"), "TOPLEFT", title, "TOPLEFT", 0, -80)
	cfg_chkNotify.tiptext = L("When checked, displays an alert message when you loot an item that begins a quest.")
	cfg_chkNotify:SetChecked(QuestClicksProfile.QuestStarterNotify)
	local orig=cfg_chkNotify:GetScript("OnClick")
	cfg_chkNotify:SetScript("OnClick", function(self)
	  orig(self)
	  if self:GetChecked() then
	    QuestClicksProfile.QuestStarterNotify = true
	  else
	    QuestClicksProfile.QuestStarterNotify = false
	  end
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
	)

    ---
    --- Growth direction buttons and labels
    ---
    
    -- Create the "Button Growth Direction" and "Direction" labels
    local lblDirectionCaption = QuestClicksOptionsLabel(QCOptionsPanel, L("Button Growth Direction:") , 20, -150, 32, 300)
    cfg_lblDirection = QuestClicksOptionsLabel(QCOptionsPanel, L(QuestClicksProfile["Direction"]), 185, -150, 32, 300)
    
    -- Create buttons for each of the possible "Growth" directions.  Each button will simply set the 
    -- QuestClicksProfile.Direction configuration variable to the appropriate value ("up","down","left","right")
    local btnUp = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 20, -150, 80, 22)
    btnUp:SetText(L("UP"))
    btnUp:SetScript("OnClick", function(self)
      QuestClicksProfile["Direction"]="up"
      cfg_lblDirection:SetText(L("up"))
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    )

    local btnDown = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 105, -150, 80, 22)
    btnDown:SetText(L("DOWN"))
    btnDown:SetScript("OnClick", function(self)
      QuestClicksProfile["Direction"]="down"
      cfg_lblDirection:SetText(L("down"))
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    )


    local btnLeft = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 190, -150, 80, 22)
    btnLeft:SetText(L("LEFT"))
    btnLeft:SetScript("OnClick", function(self)
      QuestClicksProfile["Direction"]="left"
      cfg_lblDirection:SetText(L("left"))
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    )

    local btnRight = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 275, -150, 80, 22)
    btnRight:SetText(L("RIGHT"))
    btnRight:SetScript("OnClick", function(self)
      QuestClicksProfile["Direction"]="right"
      cfg_lblDirection:SetText(L("right"))
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    ) 
    
    ---
    --- Slider for button scale
    ---
    cfg_slideScale = TKslider(QCOptionsPanel, L("Button Scale"), 1, 5, "TOPLEFT", title, "TOPLEFT", 225, -20, 150, 30)
    cfg_lblScale = QuestClicksOptionsLabel(QCOptionsPanel, QuestClicksProfile.ButtonSize, 308, -66, 30,30)
    
    cfg_slideScale:SetValueStep(1)
    cfg_slideScale:SetValue(QuestClicksProfile.ButtonSize)
    
    cfg_slideScale:SetScript("OnValueChanged", function(self)
      local i = cfg_slideScale:GetValue()
      if not i then
        i=3
      end
      cfg_lblScale:SetText(i)
      QuestClicksProfile.ButtonSize = i
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    )
    
    ---
    --- Slider for button padding
    ---
    cfg_slidePadding = TKslider(QCOptionsPanel, L("Button Padding"), 0, 20, "TOPLEFT", title, "TOPLEFT", 225, -66, 150, 30)
    cfg_lblPadding = QuestClicksOptionsLabel(QCOptionsPanel, QuestClicksProfile.Padding, 308, -105, 30,30)
    
    cfg_slidePadding:SetValueStep(1)
    cfg_slidePadding:SetValue(QuestClicksProfile.Padding)
    
    cfg_slidePadding:SetScript("OnValueChanged", function(self)
      local i = cfg_slidePadding:GetValue()
      if not i then
        i=3
      end
      cfg_lblPadding:SetText(i)
      QuestClicksProfile.Padding = i
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    )
    
	    ---
    --- Slider for buttons per Line
    ---
    cfg_slidePerLine = TKslider(QCOptionsPanel, L("Buttons Per Line"), 1, 12, "TOPLEFT", title, "TOPLEFT", 225, -105, 150, 30)
    cfg_lblPerLine = QuestClicksOptionsLabel(QCOptionsPanel, QuestClicksProfile.ButtonsPerLine, 308, -144, 30,30)
    
    cfg_slidePerLine:SetValueStep(1)
    cfg_slidePerLine:SetValue(QuestClicksProfile.ButtonsPerLine)
    
    cfg_slidePerLine:SetScript("OnValueChanged", function(self)
      local i = cfg_slidePerLine:GetValue()
      if not i then
        i=12
      end
      cfg_lblPerLine:SetText(i)
      QuestClicksProfile.ButtonsPerLine = i
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    )
	
    ---
    --- List Management Buttons
    ---
    local lblExclusions = QuestClicksOptionsLabel(QCOptionsPanel, L("Excludes"), 240, -200, 32, 300)
    local lblExtras = QuestClicksOptionsLabel(QCOptionsPanel, L("Extras"), 240, -275, 32, 300)

    local btnAddExclude = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 225, -200,  80, 22)
    btnAddExclude:SetWidth(40)
    btnAddExclude:SetHeight(40)
    btnAddExclude.tiptext = L("Drop an item here to add to the Exclusion list")
    btnAddExclude:SetNormalTexture("Interface\\Addons\\QuestClicks\\AddExcludeButton.tga")
    btnAddExclude:SetPushedTexture("Interface\\Addons\\QuestClicks\\AddExcludeButton.tga")
    btnAddExclude:SetHighlightTexture("Interface\\Addons\\QuestClicks\\AddExcludeButton.tga")
    btnAddExclude:SetDisabledTexture("Interface\\Addons\\QuestClicks\\AddExcludeButton.tga")
    btnAddExclude:GetNormalTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExclude:GetPushedTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExclude:GetHighlightTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExclude:GetDisabledTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExclude:GetHighlightTexture():SetBlendMode("ADD")
    btnAddExclude:SetScript("OnClick", function(self)
      local infotype, itemid, itemlink = GetCursorInfo()
      if infotype == "item" then
        QuestClicks_SlashHandler("exclude " .. itemlink)
        ClearCursor()
      else
        QuestClicks_DisplayMessage(L("You must have an item on your cursor when clicking this button."))
      end
    end
    ) 

    local btnRemoveExclude = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 335, -200,  80, 22)
    btnRemoveExclude:SetWidth(40)
    btnRemoveExclude:SetHeight(40)
    btnRemoveExclude.tiptext = L("Drop an item here to remove it from the Exclusion list")
    btnRemoveExclude:SetNormalTexture("Interface\\Addons\\QuestClicks\\RemoveExcludeButton.tga")
    btnRemoveExclude:SetPushedTexture("Interface\\Addons\\QuestClicks\\RemoveExcludeButton.tga")
    btnRemoveExclude:SetHighlightTexture("Interface\\Addons\\QuestClicks\\RemoveExcludeButton.tga")
    btnRemoveExclude:SetDisabledTexture("Interface\\Addons\\QuestClicks\\RemoveExcludeButton.tga")
    btnRemoveExclude:GetNormalTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExclude:GetPushedTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExclude:GetHighlightTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExclude:GetDisabledTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExclude:GetHighlightTexture():SetBlendMode("ADD")
    btnRemoveExclude:SetScript("OnClick", function(self)
      local infotype, itemid, itemlink = GetCursorInfo()
      if infotype == "item" then
        QuestClicks_SlashHandler("unexclude " .. itemlink)
        ClearCursor()
      else
        QuestClicks_DisplayMessage(L("You must have an item on your cursor when clicking this button."))
      end
    end
    ) 
    
    local btnExclude = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 270, -200, 80, 22)
    btnExclude:SetWidth(60)
    btnExclude.tiptext = L("List all items in the Excludes list")
    btnExclude:SetText(L("List"))
    btnExclude:SetScript("OnClick", function(self)
      QuestClicks_SlashHandler("list")
    end
    ) 

    local btnClearExcludes = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 270, -225, 80, 22)
    btnClearExcludes:SetWidth(60)
    btnClearExcludes.tiptext = L("Remove all items from the Excludes List")
    btnClearExcludes:SetText(L("Clear"))
    btnClearExcludes:SetScript("OnClick", function(self)
      QuestClicks_SlashHandler("clearlist")
    end
    ) 
    
    local btnAddExtra = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 225, -275,  80, 22)
    btnAddExtra:SetWidth(40)
    btnAddExtra:SetHeight(40)
    btnAddExtra.tiptext = L("Drop an item here to add to the Extras list")
    btnAddExtra:SetNormalTexture("Interface\\Addons\\QuestClicks\\AddExtraButton.tga")
    btnAddExtra:SetPushedTexture("Interface\\Addons\\QuestClicks\\AddExtraButton.tga")
    btnAddExtra:SetHighlightTexture("Interface\\Addons\\QuestClicks\\AddExtraButton.tga")
    btnAddExtra:SetDisabledTexture("Interface\\Addons\\QuestClicks\\AddExtraButton.tga")
    btnAddExtra:GetNormalTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExtra:GetPushedTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExtra:GetHighlightTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExtra:GetDisabledTexture():SetTexCoord(0, 1, 0, 1)
    btnAddExtra:GetHighlightTexture():SetBlendMode("ADD")
    btnAddExtra:SetScript("OnClick", function(self)
      local infotype, itemid, itemlink = GetCursorInfo()
      if infotype == "item" then
        QuestClicks_SlashHandler("addextra " .. itemlink)
        ClearCursor()
      else
        print("!")
        QuestClicks_DisplayMessage(L("You must have an item on your cursor when clicking this button."))
      end
    end
    ) 
    
    local btnRemoveExtra = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 335, -275,  80, 22)
    btnRemoveExtra:SetWidth(40)
    btnRemoveExtra:SetHeight(40)
    btnRemoveExtra.tiptext = L("Drop an item here to remove it from the Extras list")
    btnRemoveExtra:SetNormalTexture("Interface\\Addons\\QuestClicks\\RemoveExtraButton.tga")
    btnRemoveExtra:SetPushedTexture("Interface\\Addons\\QuestClicks\\RemoveExtraButton.tga")
    btnRemoveExtra:SetHighlightTexture("Interface\\Addons\\QuestClicks\\RemoveExtraButton.tga")
    btnRemoveExtra:SetDisabledTexture("Interface\\Addons\\QuestClicks\\RemoveExtraButton.tga")
    btnRemoveExtra:GetNormalTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExtra:GetPushedTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExtra:GetHighlightTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExtra:GetDisabledTexture():SetTexCoord(0, 1, 0, 1)
    btnRemoveExtra:GetHighlightTexture():SetBlendMode("ADD")
    btnRemoveExtra:SetScript("OnClick", function(self)
      local infotype, itemid, itemlink = GetCursorInfo()
      if infotype == "item" then
        QuestClicks_SlashHandler("removeextra " .. itemlink)
        ClearCursor()
      else
        QuestClicks_DisplayMessage(L("You must have an item on your cursor when clicking this button."))
      end
    end
    ) 
    
    local btnExtras = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 270, -275, 80, 22)
    btnExtras:SetWidth(60)
    btnExtras.tiptext = L("List all items in the Extras list")
    btnExtras:SetText(L("List"))
    btnExtras:SetScript("OnClick", function(self)
      QuestClicks_SlashHandler("extras")
    end
    ) 

    local btnClearExtras = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 270, -300, 80, 22)
    btnClearExtras:SetWidth(60)
    btnClearExtras.tiptext = L("Remove all items from the Extras List")
    btnClearExtras:SetText(L("Clear"))
    btnClearExtras:SetScript("OnClick", function(self)
      QuestClicks_SlashHandler("clearextras")
    end
    ) 
    
    -- Reset the position of the QuestClicks anchor frame to "CENTER", 0, 0
    local btnResetPos = TKbutton(QCOptionsPanel, "TOPLEFT", title, "TOPLEFT", 0, -105, 80, 22)
    btnResetPos:SetWidth(120)
    btnResetPos.tiptext = L("Reset Anchor Position")
    btnResetPos:SetText(L("Reset Position"))
    btnResetPos:SetScript("OnClick", function(self)
      QuestClicksProfile["Anchor"] = "CENTER"
      QuestClicksProfile["X"] = 0;
      QuestClicksProfile["Y"] = 0;
      QuestClicksTitleFrame:ClearAllPoints();
      QuestClicksTitleFrame:SetPoint("CENTER", UIParent, QuestClicksProfile["Anchor"], QuestClicksProfile["X"], QuestClicksProfile["Y"]);
      QuestClicksProfile["AnchorHidden"] = false;
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
      QuestClicks_DisplayMessage(L("Location reset to screen center."));
    end
    )    
    
    ---
    --- Check boxes for each defined category
    ---
    local lblCategories = QuestClicksOptionsLabel(QCOptionsPanel, L("Display conditional buttons for:"), 20, -200, 32, 300)

    -- Create a check box for each entry in the QC_CategoryNames table.  Each checkbox is offset from the starting location (QC_ConfigChecksX, QC_ConfigChecksY)
    -- by a fixed (QC_ConfigChecksVerticalSpacing) value	
	for i=1,#QC_CategoryNames do
	  QC_ConfigCheckboxes[i] = TKcheckbox(QCOptionsPanel, 26,  QC_CategoryNames[i], "TOPLEFT", title, "TOPLEFT", QC_ConfigChecksX, QC_ConfigChecksY + (QC_ConfigChecksVerticalSpacing * i))
	  QC_ConfigCheckboxes[i]:SetChecked(QuestClicksProfile.Categories[QC_CategoryNames[i]])
	  local orig=QC_ConfigCheckboxes[i]:GetScript("OnClick")
	  -- Empty the clause list so we can start with a blank slate
	  QC_ConfigCheckboxes[i]:SetScript("OnClick", function(self)
	    orig(self)
		if self:GetChecked() then
		  QuestClicksProfile.Categories[QC_CategoryNames[i]] = true
		else
		  QuestClicksProfile.Categories[QC_CategoryNames[i]] = false
		end
		-- If we enabled or disabled a category, we need to clear the caches
		-- otherwise items won't show/disapper until a UI reload.
		QC_UseClauseCache = {}
		QC_ClauseCache = {}
        QC_QuestClauseCache = {}
        QC_ArtifactClauseCache = {}
		QC_ToyPetClauseCache = {}
		QC_GarrisonClauseCache = {}
		QC_RecipeClauseCache = {}
        QC_EvalCache = {}
        QC_QuestStarterCache = {}
		QuestClicksUpdateClauseList()
		QuestClicksUpdateButtons()
	  end
	  )
	end
    
  end
end

-- Called when the configuration page needs to be updated with the current values (usually changed by a slash command)
function QuestClicksUpdateConfigPage()
  -- If the config page hasn't been loaded yet, none of these objects will exist
  if ConfigPageLoaded == true then
    cfg_chkEnabled:SetChecked(QuestClicksProfile["Enabled"])
    cfg_chkAnchor:SetChecked(QuestClicksProfile.AnchorHidden)
    cfg_chkLock:SetChecked(QuestClicksProfile.Locked)
    cfg_lblDirection:SetText(L(QuestClicksProfile["Direction"]))
    cfg_slideScale:SetValue(QuestClicksProfile.ButtonSize)
    cfg_lblScale:SetText(QuestClicksProfile.ButtonSize)
  end
end

function QuestClicksUpdateClauseList()
  QuestClicksClauseList = {}
  for i=1,#QC_CategoryNames do
    if (QuestClicksProfile.Categories[QC_CategoryNames[i]] == true) then
	  --QuestClicks_DisplayMessage ("Adding...")
	  QuestClicksAddClauses(QC_CategoryNames[i])
	else
	  --QuestClicks_DisplayMessage ("Removing...")
	  QuestClicksRemoveClauses(QC_CategoryNames[i])
	end
  end
end

---
---
--- Code to handle Slash Commands
---
--- Note: Slash commands are generally no longer the preferred way to configure QuestClicks, and many of the newer options are
---       not configurable by slash commands at all.  Please use the interface configuration GUI.
---

-- Code from WOWWiki to pull apart the text of a slash command into parameters
local function QuestClicks_GetCmd(msg)
    if msg then
      local a,b,c=strfind(msg, "(%S+)");
      if a then
        return c, strsub(msg, b+2);
      else    
        return "";
      end
    end
end
 
local function QuestClicks_GetArgument(msg)
    if msg then
      local a,b=strfind(msg, "=");
      if a then
        return strsub(msg,1,a-1), strsub(msg, b+1);
      else    
        return "";
      end
    end
end

-- Display a short message explaining the usage of the /QuestClicks slash command
local function QuestClicks_DisplayHelp()
    DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFQuestClicks:|r Available Commands - help | status | enable | disable | direction (up,down,left,right) | update")
    DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFQuestClicks:|r Commands (cont.)   - exclude (itemlink) | unexclude (itemlink) | list | clearlist")
    DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFQuestClicks:|r Commands (cont.)   - scale (1 | 2 | 3 | 4 | 5) | lock | anchor")
    DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFQuestClicks:|r Commands (cont.)   - addextra (itemlink) | removeextra (itemlink) | extras | clearextras ")
end

-- Display a message in the Chat Window in a nice, consistant format that lets the user know where the message is coming from.
function QuestClicks_DisplayMessage(msg)
    DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFQuestClicks:|r " .. msg)
end

-- Just returns "enabled" or "disabled" depending on the value of invalue
local function QuestClicks_GetStatusText(invalue)
    if invalue == true then
      return "enabled"
    else
      return "disabled"
    end
end

-- Process slash commands and dispastch them to appropriate subhandlers
function QuestClicks_SlashHandler(msg)
    local Cmd, SubCmd = QuestClicks_GetCmd(msg);
    local processed = 0;
    
    -- Display the Help Message
    if (Cmd == "help") then
    processed = 1;
      QuestClicks_DisplayHelp();
    end
    
    -- Enable the QuestClicks addon
    if (Cmd == "enable") then
      processed = 1;
      QuestClicksProfile["Enabled"] = true;      
      QuestClicksTitleFrame:Show()
      QuestClicks_DisplayMessage(L("Addon is now Enabled"));
    end

    -- Disable the QuestClicks addon    
    if (Cmd == "disable") then
      processed = 1;
      QuestClicksProfile["Enabled"] = false;
      QuestClicksTitleFrame:Hide()
      QuestClicks_DisplayMessage(L("Addon is now Disabled"));
    end
    
    -- The "Direction" command allows the user ot alter the layout of
    -- the buttons relative to the anchor frame.
    if (Cmd == "direction") or (Cmd == "dir") then
      if SubCmd == "up" or SubCmd == L("up") then
        processed = 1
        QuestClicksProfile["Direction"]="up"
      end
      if SubCmd == "down" or SubCmd == L("down")  then
        processed = 1
        QuestClicksProfile["Direction"]="down"
      end
      if SubCmd == "left"  or SubCmd == L("left") then
        processed = 1
        QuestClicksProfile["Direction"]="left"
      end
      if SubCmd == "right"  or SubCmd == L("right") then
        processed = 1
        QuestClicksProfile["Direction"]="right"
      end
      if processed == 1 then
        QuestClicks_DisplayMessage(L("Grow Direction now set to ") .. L(QuestClicksProfile["Direction"]))
        QuestClicksRepositionButtons()
      end
    end
    
    if (Cmd == "size") or (Cmd == "buttonsize") or (Cmd == "scale") or (Cmd == "buttonscale") then
      if (SubCmd) then
        if QuestClicksButtonSizes[tonumber(SubCmd)] then
          QuestClicksProfile.ButtonSize = tonumber(SubCmd)
          QuestClicksRepositionButtons()
          processed = 1
        end
      end
    end
    
	if (Cmd == "eval") then
	  if (SubCmd) then
	    local iName, iLink = GetItemInfo(SubCmd)
		if iLink then
		  processed = 1
		  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(iLink) 
          local itemSpell, itemSpellRank = GetItemSpell(iLink)		  
		  local result = QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, true)	
          if (result == true) then
		    QuestClicks_DisplayMessage("Item " .. itemLink .. " appears to be a QuestClicks item.")
	      else
		    QuestClicks_DisplayMessage("Item " .. itemLink .. " does not appear to be a QuestClicks item.")
		  end
		end
	  end	  
	end

    -- The "exclude" command adds an item to the exclusion list.
    -- Items on the Exclusion list won't have buttons genereated for them
    if (Cmd == "exclude") then
      if (SubCmd) then
        local iName, iLink = GetItemInfo(SubCmd)
        if iLink then
          processed = 1
          local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", iLink)
          QuestClicksProfile.Excludes[itemId]="X"
		  QC_EvalCache[iLink] = "X"
          QuestClicks_DisplayMessage(L("Adding Item : ") .. iLink .. L(" to exclude list."))
          QuestClicksUpdateButtons()
        end
      end
    end
    
    -- The "unexclude" command removes an item from the exclude list
    if (Cmd == "unexclude") then
      if (SubCmd) then
        local iName, iLink = GetItemInfo(SubCmd)
        if iLink then
          processed = 1
          local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", iLink)
          QuestClicksProfile.Excludes[iLink]=nil
          QuestClicksProfile.Excludes[itemId]=nil
		  QC_EvalCache[iLink] = "X"
          QuestClicks_DisplayMessage(L("Removing Item : ") .. iLink .. L(" from exclude list."))
          QuestClicksUpdateButtons()
        end
      end
    end
    
    -- The "exclude" command adds an item to the exclusion list.
    -- Items on the Exclusion list won't have buttons genereated for them
    if (Cmd == "addextra") or (Cmd == "extra") or (Cmd == "include")  then
      if (SubCmd) then
        local iName, iLink = GetItemInfo(SubCmd)
        if iLink then
          processed = 1
          local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", iLink)
          QuestClicksProfile.Includes[itemId]="X"
		  QC_EvalCache[iLink] = "X"
          QuestClicks_DisplayMessage(L("Adding Item : ") .. iLink .. L(" to inclusion list."))
          QuestClicksUpdateButtons()
        end
      end
    end
	
	if (Cmd == "iteminfo") then
	  if (SubCmd) then
	    local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(SubCmd);
		QuestClicks_DisplayMessage ("Name: "..sName)
		QuestClicks_DisplayMessage ("ItemID: " .. sLink:match("item:(%d+):"))
		QuestClicks_DisplayMessage ("sLink: "..sLink)
		QuestClicks_DisplayMessage ("iRarity: "..iRarity)
		QuestClicks_DisplayMessage ("iLevel: "..iLevel)
		QuestClicks_DisplayMessage ("iMinLevel: "..iMinLevel)
		QuestClicks_DisplayMessage ("sType: "..sType)
		QuestClicks_DisplayMessage ("sSubType: "..sSubType)
		QuestClicks_DisplayMessage ("iStackCount: "..iStackCount)	
        QuestClicksScanningTooltip:ClearLines()
        QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
        QuestClicksScanningTooltip:SetHyperlink(SubCmd)
        QuestClicksScanningTooltip:Show()
        
        -- Scan each line of the tooltip looking for "This Item Begins a Quest"
        local tipTextLeft = QuestClicksScanningTooltip:GetName().."TextLeft"
        for i=1,QuestClicksScanningTooltip:NumLines() do
          text = _G["QuestClicksScanningTooltipTextLeft"..i]:GetText() or "";
          QuestClicks_DisplayMessage ("Tool Tip Line "..i ..": " ..text)
        end

		processed = 1
	  end
	end
	
	if (Cmd == "findrelics") then
	  local relicFilter = ""
	  if (SubCmd) then
	    relicFilter = SubCmd 
	  end
	  for bag = 1,7 do
		if QC_ScanBags[bag] == true then
		  for slot = 1,GetContainerNumSlots(QC_BagIDs[bag]) do
			local item = GetContainerItemLink(QC_BagIDs[bag],slot) 
			if item then
			  -- Get information about the item
			  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item) 
			  local itemSpell, itemSpellRank = GetItemSpell(item)
			  
			  QuestClicksScanningTooltip:ClearLines()
              QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
              QuestClicksScanningTooltip:SetHyperlink(itemLink)
              QuestClicksScanningTooltip:Show()
              local tipTextLeft = QuestClicksScanningTooltip:GetName().."TextLeft"
              for i=1,QuestClicksScanningTooltip:NumLines() do
                text = _G["QuestClicksScanningTooltipTextLeft"..i]:GetText() or "";
				if string.find(text, relicFilter .. " Artifact Relic") ~= nil then
                  QuestClicks_DisplayMessage ("Found relic: " .. itemLink)
				end
              end
			end
		  end
		end
	  end	
	  processed = 1
	end

	if (Cmd == "clauselist") then
	  QuestClicks_DisplayMessage ("Listing Clauses:")
	  QuestClicks_DisplayMessage(#QuestClicksClauseList)
      for k, v in pairs(QuestClicksClauseList) do
	    tv="false"
		if (v) then tv="true" end
		QuestClicks_DisplayMessage (k .. "=" .. tv)
		processed = 1
  	  end
    end

    -- The "unexclude" command removes an item from the exclude list
    if (Cmd == "removeextra") or (Cmd == "delextra") then
      if (SubCmd) then
        local iName, iLink = GetItemInfo(SubCmd)
        if iLink then
          processed = 1
          local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", iLink)
          QuestClicksProfile.Includes[iLink]=nil
          QuestClicksProfile.Includes[itemId]=nil
		  QC_EvalCache[iLink] = "X"
          QuestClicks_DisplayMessage(L("Removing Item : ") .. iLink .. L(" from inclusion list."))
          QuestClicksUpdateButtons()
        end
      end
    end
    
    -- The "lock" command toggles the frame from being draggable
    if (Cmd == "lock") then
      if QuestClicksProfile.Locked == true then
        QuestClicksProfile.Locked = false
      else
        QuestClicksProfile.Locked = true
      end
      processed = 1
      QuestClicks_DisplayMessage(L("Bar locking is ") .. QuestClicks_GetStatusText(QuestClicksProfile.Locked))
    end
    
    -- The "combineable" command toggles the frame from being draggable
    if (Cmd == "combine") or (Cmd == "combinable") or (Cmd == "collapsible") or (Cmd == "collapse") then
      if QuestClicksProfile.Collapsibles== true then
        QuestClicksProfile.Collapsibles = false
      else
        QuestClicksProfile.Collapsibles = true
      end
      processed = 1
      QuestClicksUpdateButtons()
      QuestClicks_DisplayMessage(L("Buttons for Collapsabale Items is ") .. QuestClicks_GetStatusText(QuestClicksProfile.Collapsibles))
    end
    
    -- The "anchor" command toggles hiding the dragging anchor, in effect also locking the bar.
    if (Cmd == "anchor") then
    processed = 1
      if QuestClicksProfile.AnchorHidden == false then
        QuestClicksProfile.AnchorHidden = true
        QuestClicks_DisplayMessage(L("Anchor Frame now Hidden"))
      else
        QuestClicksProfile.AnchorHidden = false
        QuestClicks_DisplayMessage(L("Anchor Frame now Shown"))
      end
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
    end
    
    -- The "clearexclude" command removes all items from the Exclude list
    if (Cmd == "clearexclude") or (Cmd == "clear") or (Cmd == "clearlist") then
      processed = 1
      QuestClicksProfile.Excludes = {}
      QuestClicks_DisplayMessage(L("Exclusion list cleared."));
      QuestClicksUpdateButtons()
    end
    
    -- The "clearextra" command removes all items from the Exclude list
    if (Cmd == "clearextra") or (Cmd == "clearextras") then
      processed = 1
      QuestClicksProfile.Includes = {}
      QuestClicks_DisplayMessage(L("Extra list cleared."));
      QuestClicksUpdateButtons()
    end
    
    -- The "List" command will show all excluded items
    if (Cmd == "extras") or (Cmd == "extraslist") then
      processed = 1
      local count = 0
      for k,v in pairs(QuestClicksProfile.Includes) do
        local iname, ilink = GetItemInfo(k)
		if not (ilink==nil) then
          QuestClicks_DisplayMessage(L("Extra item : ") .. ilink .. ".");
          count = count + 1
        end
      end
      if count == 0 then
        QuestClicks_DisplayMessage(L("Extra list is empty."));
      end
    end
    
    -- The "extras" command will show all excluded items
    if (Cmd == "list") or (Cmd == "excludelist") then
      processed = 1
      local count = 0
      for k,v in pairs(QuestClicksProfile.Excludes) do
        local iname, ilink = GetItemInfo(k)
        if not (ilink==nil) then
          QuestClicks_DisplayMessage(L("Excluding item : ") .. ilink .. ".");
          count = count + 1   
	    end
      end
      if count == 0 then
        QuestClicks_DisplayMessage(L("Exclusion list is empty."));
      end
    end
    
    -- The "resetpos" command resets the location of the anchor to the center of the screen
    if (Cmd == "resetpos") then
      processed = 1;
      QuestClicksProfile["Anchor"] = "CENTER"
      QuestClicksProfile["X"] = 0;
      QuestClicksProfile["Y"] = 0;
      QuestClicksTitleFrame:ClearAllPoints();
      QuestClicksTitleFrame:SetPoint("CENTER", UIParent, QuestClicksProfile["Anchor"], QuestClicksProfile["X"], QuestClicksProfile["Y"]);
      QuestClicksProfile["AnchorHidden"] = false;
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
      QuestClicks_DisplayMessage(L("Location reset to screen center."));
    end
    
    -- Show the status of all settable variables for the QuestClicks addon
    if (Cmd == "status") then
      processed = 1;
      QuestClicks_DisplayMessage(L("Addon is ") .. QuestClicks_GetStatusText(QuestClicksProfile["Enabled"]));
      QuestClicks_DisplayMessage(L("Grow Direction is ") .. QuestClicksProfile["Direction"]);
      QuestClicks_DisplayMessage(L("Button Scale is ") .. QuestClicksProfile.ButtonSize);
      QuestClicks_DisplayMessage(L("Show Collapsible Items is ") .. QuestClicks_GetStatusText(QuestClicksProfile.Collapsibles));
    end

    -- Shouldn't have to use this, but it is here for testing purposes.
    -- Just runs the QuestClicksUpdateButtons() function directly
    if (Cmd == "update") then
      processed = 1
      QuestClicksUpdateButtons()
    end

    -- If none of the command above matched the player's input, show the help
    if processed==0 then
      InterfaceOptionsFrame_OpenToCategory(QCOptionsPanel)
    else
      QuestClicksUpdateConfigPage()
    end
end

-- Hides the QuestClicks Anchor Frame.
function QuestClicksHideAnchor()
    QuestClicksTitleFrame:SetBackdrop(QuestClicksEmptyBackdrop)
    QuestClicksTitleLabel:SetText("")
end

-- Shows the QuestClicksAnchorFrame
function QuestClicksShowAnchor()
    QuestClicksTitleFrame:SetBackdrop(QuestClicksSaveBackdrop)
    QuestClicksTitleLabel:SetText("Clicks")
end


---
---
--- Button Creation and Positioning
---
---

-- Repositions each of the "sub buttons" based on the direction variable
function QuestClicksRepositionButtons()
    -- Check to see if we need to add padding between the buttons
    local padding = 0
	local rowcounter = 0
	local rowoffsetX = 0
	local rowoffsetY = 0
	local ButtonsPerLine = QuestClicksProfile.ButtonsPerLine
	if (ButtonsPerLine == nil) then ButtonsPerLine=QuestClicksMax end
	
    if QuestClicksProfile.Padding then padding=QuestClicksProfile.Padding end
    
    -- We need the anchor frame to position our buttons relative to it.
    QuestClicksShowAnchor()
    
    -- Defaults
    local xMod = 0
    local yMod = (-1 * QuestClicksButtonSizes[QuestClicksProfile.ButtonSize]) - padding
    local xOfs = 0
    local yOfs = 0
    local RelativePoint = "TOP"
    
    -- Modify some of the above defaults based on the growth direction of the bar
    if QuestClicksProfile["Direction"] == "down" then
      xMod = 0
      yMod = (-1 * QuestClicksButtonSizes[QuestClicksProfile.ButtonSize]) - padding
      AnchorPoint = "BOTTOM"
	  rowoffsetX = (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding)
	  rowoffsetY = (ButtonsPerLine * (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding))
    end
    if QuestClicksProfile["Direction"] == "up" then
      xMod = 0
      yMod = QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding
      yOfs = QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding
      AnchorPoint = "TOP"
	  rowoffsetX = (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding)
	  rowoffsetY = -1 * (ButtonsPerLine * (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding))
    end 
    if QuestClicksProfile["Direction"] == "right" then
      xMod = QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding
      yMod = 0
      xOfs = -23
      yOfs = -10
      AnchorPoint = "RIGHT"
	  rowoffsetX = -1 * (ButtonsPerLine * (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding))
	  rowoffsetY = -1 * (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding)
    end
    if QuestClicksProfile["Direction"] == "left" then
      xMod = -1 * QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] - padding
      yMod = 0
      xOfs = 23
      yOfs = -10
      AnchorPoint = "LEFT"
	  rowoffsetX = (ButtonsPerLine * (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding))
  	  rowoffsetY = -1 * (QuestClicksButtonSizes[QuestClicksProfile.ButtonSize] + padding)
   end
    
    -- Loop through each button and set their position and size according to the values determined above
    for i=1, QuestClicksMax do
	  local rowNumber = math.floor(rowcounter / ButtonsPerLine)
      QuestClicksSubframes[i]:SetWidth(QuestClicksButtonSizes[QuestClicksProfile.ButtonSize])
      QuestClicksSubframes[i]:SetHeight(QuestClicksButtonSizes[QuestClicksProfile.ButtonSize])
      QuestClicksSubframes[i]:SetPoint("TOP", QuestClicksTitleFrame, AnchorPoint, (((i-1) * xMod) + xOfs) + (rowNumber * rowoffsetX), (((i-1) * yMod) + yOfs) + (rowNumber * rowoffsetY) )
	  rowcounter = rowcounter + 1
    end
    
    -- If the anchor should be hidden, rehide it.
    if QuestClicksProfile.AnchorHidden == true then
      QuestClicksHideAnchor()
    end
    
    -- If Button Facade is enabled, skin the buttons
    if LBF then
      if QCBF_Group then
        QCBF_Group:Skin(QuestClicksProfile.ButtonFacade.Skin, QuestClicksProfile.ButtonFacade.Gloss, QuestClicksProfile.ButtonFacade.Backdrop)
      end
    end
end

function QuestClicksUpdateQuantities()
  for i=1,12 do
    if QuestClicksTypes[i] == "I" then
      local itemCount = GetItemCount(QuestClicksLinks[i])
      if itemCount > 1 then
        QuestClicksSubframes[i].ItemCount:SetTextColor(1,0.9,0)
        QuestClicksSubframes[i].ItemCount:SetText(itemCount)
      else
      -- If the item is a quest starter, display a green asterisk (*) instead of a number
        if QC_QuestStarterCache[QuestClicksLinks[i]] then
          QuestClicksSubframes[i].ItemCount:SetTextColor(0,1,0)
          QuestClicksSubframes[i].ItemCount:SetText("*")
        else
          QuestClicksSubframes[i].ItemCount:SetText("")
        end
      end
    end
  end
end

-- Hide buttons in the QuestClicks bar, starting at "bnum" and going until the normal max (12).
-- Note that this could be done better... the 12 shouldn't be hardcoded here.
function QuestClicksHideButtons(bnum)
    for i=bnum,12 do
      QuestClicksSubframes[i]:Hide()
      QuestClicksSubframes[i]:SetText("")
      QuestClicksLinks[i]=nil
      QuestClicksMacros[i]=nil
      QuestClicksTypes[i]=nil
    end
end

function QuestClicksAddClauses(CategoryName)
  if CategoryName == L("Additions") then
  end

  if CategoryName == L("Combines") then
  end

  if CategoryName == L("Trades") then
  end

  if CategoryName == L("Keys") then
  end
  
  if CategoryName == L("Quest Starters") then
    QuestClicksClauseList[ITEM_STARTS_QUEST] = true
  end

  if CategoryName == L("Artifact Boosters") then
    QuestClicksClauseList[L("Use: Grants .* Artifact Power to your currently equipped Artifact")] = true
  end

  if CategoryName == L("Toys and Pets") then
    QuestClicksClauseList[L("Use: Adds this toy to your Toy Box")] = true
	QuestClicksClauseList[L("Teaches you how to summon ")] = true
  end
  
  if CategoryName == L("Garrison Rewards") then
    QuestClicksClauseList[L("Create a soulbound item appropriate for your loot specialization")] = true
	QuestClicksClauseList[L("Use: Open your Followers page and use this item to grant")] = true
	QuestClicksClauseList[L("Use: Increases a Champion's")] = true
	QuestClicksClauseList[L("Use: Gain .* Garrison Resources")] = true
	QuestClicksClauseList[L("Use: Create .* follower weapon enhancement")] = true
	QuestClicksClauseList[L("Use: Upgrade a follower's ")] = true
  end

  if CategoryName == L("Recipes") then
  end

  if CategoryName == L("Lootable Containers") then
	QuestClicksClauseList[L("Use: Open the ")] = true
	QuestClicksClauseList[L("Blingtron .* Gift Package")] = true
	QuestClicksClauseList[L("Greater Tribute of the Broken Isles")] = true
  end

  if CategoryName == L("Oneshot Boosts") then
	QuestClicksClauseList[L("Use: Gain ...? Ancient Mana.")] = true
	QuestClicksClauseList[L("Use: Increase the maximum amount of Ancient Mana you can carry")] = true
	QuestClicksClauseList[L("Use: Gain ...? Nethershards.")] = true
	QuestClicksClauseList[L("Use: Increase.* reputation with .* by .*")] = true
	QuestClicksClauseList[L("Use: Grants .* reputation with .*")] = true
	QuestClicksClauseList[L("This effect is permanent and will take effect the next time you enter")] = true
  end
  
  if CategoryName == L("Engineer Tools") then
  end
  
end

function QuestClicksRemoveClauses(CategoryName)
  if CategoryName == L("Additions") then
  end

  if CategoryName == L("Combines") then
  end

  if CategoryName == L("Trades") then
  end

  if CategoryName == L("Keys") then
  end
  
  if CategoryName == L("Quest Starters") then
    QuestClicksClauseList[ITEM_STARTS_QUEST] = true
  end

  if CategoryName == L("Artifact Boosters") then
    QuestClicksClauseList[L("Use: Grants .* Artifact Power to your currently equipped Artifact")] = false
  end

  if CategoryName == L("Toys and Pets") then
    QuestClicksClauseList[L("Use: Adds this toy to your Toy Box")] = false
	QuestClicksClauseList[L("Teaches you how to summon ")] = false
  end
  
  if CategoryName == L("Garrison Rewards") then
    QuestClicksClauseList[L("Create a soulbound item appropriate for your loot specialization")] = false
	QuestClicksClauseList[L("Use: Open your Followers page and use this item to grant")] = false
	QuestClicksClauseList[L("Use: Increase a Champion's")] = false
	QuestClicksClauseList[L("Use: Gain .* Garrison Resources")] = false
	QuestClicksClauseList[L("Use: Create .* follower weapon enhancement")] = false
	QuestClicksClauseList[L("Use: Upgrade a follower's ")] = false
  end

  if CategoryName == L("Recipes") then
  end

  if CategoryName == L("Lootable Containers") then
	QuestClicksClauseList[L("Use: Open the ")] = false
	QuestClicksClauseList[L("Blingtron .* Gift Package")] = false
	QuestClicksClauseList[L("Greater Tribute of the Broken Isles")] = false
  end
  
  if CategoryName == L("Oneshot Boosts") then
	QuestClicksClauseList[L("Use: Gain ...? Ancient Mana.")] = false
	QuestClicksClauseList[L("Use: Increase the maximum amount of Ancient Mana you can carry")] = false
	QuestClicksClauseList[L("Use: Gain ...? Nethershards.")] = false
	QuestClicksClauseList[L("Use: Increase.* reputation with .* by .*")] = false
	QuestClicksClauseList[L("Use: Grants .* reputation with .*")] = false
	QuestClicksClauseList[L("This effect is permanent and will take effect the next time you enter")] = false
  end

  if CategoryName == L("Engineer Tools") then
  end
end

-- Scan an item's tooltip to see if it contains "This Item Begins a Quest"
function QuestClicksIsStartQuestItem(itemLink)
  -- Default to false
  local isQuestItem=false
  
  -- Make sure we have something passed as itemLink
  if itemLink then
    -- Check QC_QuestStarterCache to see if we have already scanned this item.
    if QC_QuestStarterCache[itemLink] then
      -- If so, use the cached result of the scan
      isQuestItem = QC_QuestStarterCache[itemLink]
    else
      -- Check to make sure itemLink is a valid item.  GetItemInfo returns "nil" if it isn't.
      if GetItemInfo(itemLink) then
        -- Set up the Scanning Tooltip
        QuestClicksScanningTooltip:ClearLines()
        QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
        QuestClicksScanningTooltip:SetHyperlink(itemLink)
        QuestClicksScanningTooltip:Show()
        
        -- Scan each line of the tooltip looking for "This Item Begins a Quest"
        local tipTextLeft = QuestClicksScanningTooltip:GetName().."TextLeft"
        for i=1,QuestClicksScanningTooltip:NumLines() do
          text = _G["QuestClicksScanningTooltipTextLeft"..i]:GetText() or "";
          -- "ITEM_STARTS_QUEST" is the pre-localized constant for "This Item Begins a Quest"
          if text == ITEM_STARTS_QUEST then
            isQuestItem=true
          end
        end
        
        -- Now that we have scanned the item, cache the result in QC_QuestStarterCache
        QC_QuestStarterCache[itemLink] = isQuestItem
      end
    end
  end
  
  -- Return the result
  return isQuestItem
end

function QuestClicksItemHasUseClause(itemLink, debugprint)
  -- Default to "false"
  local HasUseClause=false
  
  -- Make sure an itemLink was passed
  if itemLink then
    -- Check to see if we have already scanned this item.
    if QC_UseClauseCache[itemLink] and debugprint==false then
      -- If so, return the results of the previous scan
      HasUseClause = QC_UseClauseCache[itemLink]
    else
      -- Otherwise, scan the tooltip
      if GetItemInfo(itemLink) then
        QuestClicksScanningTooltip:ClearLines()
        QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
        QuestClicksScanningTooltip:SetHyperlink(itemLink)
        QuestClicksScanningTooltip:Show()
        local tipTextLeft = QuestClicksScanningTooltip:GetName().."TextLeft"  
        for i=1,QuestClicksScanningTooltip:NumLines() do		
          text = _G["QuestClicksScanningTooltipTextLeft"..i]:GetText() or "";
		  if (debugprint==true) then
		    QuestClicks_DisplayMessage(text)
		  end
          if string.find(text, L("Use:")) == 1 then
			HasUseClause=true
          end
        end
        -- Cache the result of the scan in QC_UseClauseCache
        QC_UseClauseCache[itemLink] = HasUseClause
      end
    end
  end

  return HasUseClause
end

function QuestClicksCheckBlockers(itemLink)
  local AlreadyKnown = false
  local RedRequirement = false

  if (QC_BlockersCache[itemLink] ~= nil) then
    return QC_BlockersCache[itemLink]
  end
  
  if itemLink then
    if GetItemInfo(itemLink) then
      QuestClicksScanningTooltip:ClearLines()
      QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
      QuestClicksScanningTooltip:SetHyperlink(itemLink)
      QuestClicksScanningTooltip:Show()
      local tipTextLeft = QuestClicksScanningTooltip:GetName().."TextLeft"  
      for i=1,QuestClicksScanningTooltip:NumLines() do		
        local fs = _G["QuestClicksScanningTooltipTextLeft"..i]
        textR, textG, textB, textAlpha = fs:GetTextColor() 			
	    text = _G["QuestClicksScanningTooltipTextLeft"..i]:GetText() or "";
		if string.find(text, L("Already known")) ~= nil then
          AlreadyKnown = true
        end	
  		sStart,sEnd = string.find(text, "Requires ")
 		if (sStart == 1) then
 	      if textR > 0.9 and textB < 0.2 and textG < 0.2 then
			RedRequirement = true
  		  end
	    end
  	  end
    end
  end
  
  QC_BlockersCache[itemLink] = AlreadyKnown or RedRequirement
  
  return QC_BlockersCache[itemLink]
end

function QuestClicksCheckClauseList(itemLink, debugprint)
  -- Default to "false"
  local ClauseMatch=false
  local AlreadyKnown=false
  
  -- Make sure an itemLink was passed
  if itemLink then
    -- Check to see if we have already scanned this item.
    if QC_ClauseCache[itemLink] and debugprint==false then
      -- If so, return the results of the previous scan
      ClauseMatch = QC_ClauseCache[itemLink]
    else
      -- Otherwise, scan the tooltip
      if GetItemInfo(itemLink) then
        QuestClicksScanningTooltip:ClearLines()
        QuestClicksScanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE" );
        QuestClicksScanningTooltip:SetHyperlink(itemLink)
        QuestClicksScanningTooltip:Show()
        local tipTextLeft = QuestClicksScanningTooltip:GetName().."TextLeft"  
        for i=1,QuestClicksScanningTooltip:NumLines() do		
          text = _G["QuestClicksScanningTooltipTextLeft"..i]:GetText() or "";
		  for k, v in pairs(QuestClicksClauseList) do
		    if v then
		      if string.find(text, k) ~= nil then
			    ClauseMatch = true
			  end
			end
		  end		  
        end
        -- Cache the result of the scan in QC_UseClauseCache
        QC_ClauseCache[itemLink] = ClauseMatch
      end
    end
	
	return ClauseMatch and (not QuestClicksCheckBlockers(itemLink))
  end
end

function QuestClicksItemIsUnlearnedRecipie(itemLink, debugprint)
  -- Default to "false"
  local IsNewRecipe=false
  
  -- Make sure an itemLink was passed
  if itemLink then
    -- Check to see if we have already scanned this item.
    if QC_RecipeClauseCache[itemLink] then
      -- If so, return the results of the previous scan
      IsNewRecipe = QC_RecipeClauseCache[itemLink]
    else
      -- Otherwise, scan the tooltip
      if GetItemInfo(itemLink) then
	    -- Check the item type to see if this item is a recipe
	    local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemLink);
		if sType == "Recipe" then 
		  -- If the item is a recipe, assume we are going to include it unless we determine otherwise
          IsNewRecipe = not QuestClicksCheckBlockers(itemLink)
        end
        -- Cache the result of the scan in QC_RecipeClauseCache
        QC_RecipeClauseCache[itemLink] = IsNewRecipe
      end
    end
	
	return IsNewRecipe
  end
end

-- QuestClicksEvaluateItem is called for every item in the players bags/inventory.  It will use multiple criteria to
-- determine if the item should get a button.
function QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, debugprint)

    if itemLink == nil then return false end
	local wasConditionalItem = false
	
	if QC_EvalCache[itemLink] == "N" then return false end
	if QC_EvalCache[itemLink] == "Y" then return true end
 
    local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", itemLink)

    -- Assume the item WON'T get a button
    local bIncludeItem = false
	
	local bIsQuestItem = false
	
    -- If this item is a Quest Item and it triggers a spell, include it.  This is the most basic test and
    -- catches 95% of the items we are interested in.
	if (debugprint==true) then
	   QuestClicks_DisplayMessage(itemType)
	end

	if itemType=="Quest" or itemType==L("Quest") then
	  bIsQuestItem = true
	end
	
	if (debugprint==true) then
	   QuestClicks_DisplayMessage("Is it a quest item?: " .. tostring(bIsQuestItem))
	end
		
    if bIsQuestItem == true then
      if itemSpell then
        bIncludeItem = true
      end
    end
	
    -- If the item is a quest item and has a "Use" clause, include it.  Some items don't return anything for
    -- itemSpell (weirdly enough) and we scan their tooltip for "Use:" in this case.  Note that we only do this
    -- if we haven't already decided to add the item.
    if bIncludeItem == false then
	  if (debugprint==true) then
        QuestClicks_DisplayMessage("Not Including So Far... " .. itemLink .. " : " .. tostring(bIsQuestItem))
	  end
      if bIsQuestItem==true then
        if QuestClicksItemHasUseClause(itemLink, debugprint) then
          bIncludeItem = true
        end
      end
    end
                
    -- If this item is on the Includes list, include it                
    if bIncludeItem == false then
      if QuestClicksProfile.Includes[itemId] then
        bIncludeItem = true
      end
    end

    -- Check extra item categories
    -- Get the sub-table from QC_Conditionals for this item (if it exists)
	--local TestInfo = QC_Conditionals[itemName]
    
    --if not TestInfo then TestInfo = QC_Conditionals[itemId] end
    
    TestInfo = QC_Conditionals[itemId]
    
    -- If it does, lets test it.
	if TestInfo then
	  --QuestClicks_DisplayMessage("Was a Conditional Item")
	  wasConditionalItem = true
	  local TestCount = TestInfo[1]
	  local TestSkill = TestInfo[2]
	  local TestCategory = TestInfo[3]
      -- Make sure this category is enabled in the options panel
	  if QuestClicksProfile.Categories[TestCategory] == true then
        -- Assume the player has the skill necessary for this item.
	    local SkillOk = true
        -- If a required spell was specified, test it.
	    if TestSkill then
          -- GetSpellInfo returns nil if the player doesn't have that spell, otherwise it returns values.
          -- we don't care what the actual values are, only that it exists.
		  if GetSpellInfo(TestSkill) then
		    SkillOk = true
			--QuestClicks_DisplayMessage("Skill Check Passed")
		  else
		    SkillOk = false
		  end
	    end
        
        -- If the category is enabled, and the skill test is passed, check the required quantity.
        if TestCount then
		  if SkillOk == true then
            -- GetItemCount returns the number of itemLink items in the player's bags/inventory
            local itemCount = GetItemCount(itemLink)
            if itemCount >= TestCount then
              -- Make sure we don't already have a button for this item.
              if not QuestClicksCombinables[itemName] then
                -- Mark as having a button for the item and include it.
                QuestClicksCombinables[itemName] = "X"
                bIncludeItem = true
				--QuestClicks_DisplayMessage("Item Count passed")
              end
            end
		  end
		end
      end
    end
 
    -- If we haven't included this item yet, scan it's tooltip to see if it starts a quest
	if bIncludeItem == false then
	  bIncludeItem = QuestClicksCheckClauseList(itemLink)
	end
 
    if bIncludeItem == false then
	  if QuestClicksProfile.Categories[L("Recipes")] == true then
	    bIncludeItem = QuestClicksItemIsUnlearnedRecipie(itemLink)
	  end
	end

    -- If this item is on the excludes list, don't include it no matter what the above would say
    if QuestClicksProfile.Excludes[itemId] then
      bIncludeItem = false
    end
 
    if bIncludeItem == true then
	  QC_EvalCache[itemLink] = "Y"
	else
	  QC_EvalCache[itemLink] = "N"
	end
	
	if wasConditionalItem==true then
	  QC_EvalCache[itemLink] = "X"
	end
 
    return bIncludeItem
end

-- Builds a button.  The frame and UI components already exist, so we are filling in the
-- customized pieces.
function QuestClicksCreateButton(itemLink, itemName, itemTexture, ButtonNumber, ReqSkill)

    if ReqSkill == nil then
      --print("CreateButton: " .. itemLink .. ", " .. itemName .. ", " .. itemTexture .. ", " .. ButtonNumber);
	else
	  --print("CreateButton: " .. itemLink .. ", " .. itemName .. ", " .. itemTexture .. ", " .. ButtonNumber .. ", " .. ReqSkill);
	end

    -- Check the existing button information for this button to see if it is already set to what we want
    local item = QuestClicksSubframes[ButtonNumber]:GetAttribute("item*")
    --local macro = QuestClicksSubframes[ButtonNumber]:GetAttribute("macrotext*")

    local process = false

    if not item then 
      process = true 
    else
      if not (item == itemName) then
        process = true
      end
    end
    
	if ReqSkill then process = true end
	
    if process == true then
      QuestClicksSubframes[ButtonNumber]:SetAttribute("type*", "item")
      QuestClicksSubframes[ButtonNumber]:SetAttribute("item*", itemName)
      QuestClicksLinks[ButtonNumber] = itemLink
      QuestClicksTypes[ButtonNumber] = "I"

      -- If the button was created with a skill requirement, we use a macro instead of a normal item button
      if ReqSkill then
        QuestClicksSubframes[ButtonNumber]:SetAttribute("type*", "macro")
        QuestClicksSubframes[ButtonNumber]:SetAttribute("macrotext*", "/cast " .. ReqSkill .. "\n/use " .. itemName)
        QuestClicksMacros[ButtonNumber] = "/cast " .. ReqSkill .. "\n/use " .. itemName
        QuestClicksTypes[ButtonNumber] = "M"
      end
      
      -- Get the quantity to display on the button
      local itemCount = GetItemCount(itemLink)
        if itemCount > 1 then
          QuestClicksSubframes[ButtonNumber].ItemCount:SetTextColor(1,0.9,0)
          QuestClicksSubframes[ButtonNumber].ItemCount:SetText(itemCount)
        else
        -- If the item is a quest starter, display a green asterisk (*) instead of a number
          if QC_QuestStarterCache[itemLink] then
            QuestClicksSubframes[ButtonNumber].ItemCount:SetTextColor(0,1,0)
            QuestClicksSubframes[ButtonNumber].ItemCount:SetText("*")
          else
            QuestClicksSubframes[ButtonNumber].ItemCount:SetText("")
          end
        end

      QuestClicksSubframes[ButtonNumber].Icon:SetTexture(itemTexture)
    
      -- Notify of a looted quest starter item
      if QuestClicksProfile.QuestStarterNotify == true then
        if QC_QuestStarterCache[itemLink] then
          if not QC_AlertedQuestStarters[itemLink] then
            QC_AlertedQuestStarters[itemLink] = true
            QCMessage("Quest Starter Item Looted : " .. itemLink)
            PlaySound(23404) --changed to UI_AUTO_QUEST_COMPLETE = 23404 since the other is no longer in game
          end
        end
      end
    else
      -- If the button was already set up for this item, update the quantity.
      local itemCount = GetItemCount(itemLink)
      if itemCount > 1 then
        QuestClicksSubframes[ButtonNumber].ItemCount:SetTextColor(1,0.9,0)
        QuestClicksSubframes[ButtonNumber].ItemCount:SetText(itemCount)
      end
    end
    
    QuestClicksLinks[ButtonNumber] = itemLink
    QuestClicksSubframes[ButtonNumber]:Show()
end

-- Currently experimental, unused code
-- Returns the number of slots in a "bag".  Special function because we treat
-- bag #19 (which doesn't really exist) as the player's worn inventory and return
-- the number of equipable items slots on the paper doll
local function QC_GetBagSlots(bag)
  if bag == 19 then
    return 19
  else
    return GetContainerNumSlots(bag)
  end
end

-- Currently experimental, unused code
-- Returns the itemLink for an item in a "bag".  Note that as with QC_GetBagSlots we
-- treat bag 19 as the player's worn inventory and use a different API command to get
-- the link.
local function QC_GetBagItem(bag, slot)
  if bag == 19 then
    return GetInventoryItemLink("player", GetInventorySlotInfo(QuestClicksInventory[slot]))
  else
    return GetContainerItemLink(bag,slot)
  end
end

function QuestClicksUpdateButtons()
  -- Start with the first button.
  local CurrentButton = 1
  local CreatedButtons = {}
    
  QCDebugMessage("QuestClicks: UpdateButtons Called")
    
  -- Clear the QuestClicksCombinables list for this itteration of the update
  QuestClicksCombinables = {}   

  -- Determine if we should be scanning the keyring
  if (QuestClicksProfile.Categories[L("Keys")] == true) then
    QC_ScanBags[1] = true
  else
    QC_ScanBags[1] = false
  end
    
  for bag = 1,7 do
    if QC_ScanBags[bag] == true then
      for slot = 1,QC_GetBagSlots(QC_BagIDs[bag]) do
        local item = QC_GetBagItem(QC_BagIDs[bag],slot) 
        if item then
          -- Get information about the item
          local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item) 
          local itemSpell, itemSpellRank = GetItemSpell(item)

          -- Evaluate the item to see if it should be on the bar
          if QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, false) == true then
            -- If true, we can create the button if we haven't run out.
            if CurrentButton <= QuestClicksMax then
			  local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", itemLink)
              local TestInfo = QC_Conditionals[itemId]
              local TestSkill = nil
              if TestInfo then
                TestSkill = TestInfo[2]
              end
              if not (CreatedButtons[itemId] == "X") then
                QuestClicksCreateButton(itemLink, itemName, itemTexture, CurrentButton, TestSkill)
                CreatedButtons[itemId] = "X"
                CurrentButton = CurrentButton + 1
              end
            end
          end
        end
      end
    end
  end
    
  -- Skin the buttons if ButtonFacade is enabled    
  if LBF then
    if QCBF_Group then
      QCBF_Group:Skin(QuestClicksProfile.ButtonFacade.Skin, QuestClicksProfile.ButtonFacade.Gloss, QuestClicksProfile.ButtonFacade.Backdrop)
    end
  end

  -- Hide any buttons that were not set up
  QuestClicksHideButtons(CurrentButton)  
  QuestClicksUpdateQuantities()
  --UpdateCooldownFrames()
  
end

-- Look through all inventory bags and find items that should be put onto the bar.
function QuestClicksUpdateButtonsOld()

    -- Track which button we are updating
    local CurrentButton = 1
    local CreatedButtons = {}
    
    QCDebugMessage("QuestClicks: UpdateButtons Called")
    
    -- Clear the QuestClicksCombinables list for this itteration of the update
    QuestClicksCombinables = {}    
    
    -- Look for items in bags    
    for bag = 0,4 do
      for slot = 1,GetContainerNumSlots(bag) do
        local item = GetContainerItemLink(bag,slot)
        if item then
          -- Get information about the item
          local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item) 
          local itemSpell, itemSpellRank = GetItemSpell(item)

          -- Evaluate the item to see if it should be on the bar
          if QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, false) == true then
            -- If true, we can create the button if we haven't run out.
            if CurrentButton <= QuestClicksMax then
			  local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", itemLink)
              local TestInfo = QC_Conditionals[itemId]
              local TestSkill = nil
 	          if TestInfo then
	            TestSkill = TestInfo[2]
              end
              if not CreatedButtons[itemLink] then
                QuestClicksCreateButton(itemLink, itemName, itemTexture, CurrentButton, TestSkill)
                CreatedButtons[itemLink] = true
                CurrentButton = CurrentButton + 1
              end
            end
          end
        end
      end
    end
	
    -- Check the Key Ring if that option is enabled in the config.
    if QuestClicksProfile.Categories[L("Keys")] == true then
	  if 1 == 1 then
	    bag=KEYRING_CONTAINER
	    for slot = 1,GetContainerNumSlots(bag) do
          local item = GetContainerItemLink(bag,slot)
          if item then
            local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item) 
            local itemSpell, itemSpellRank = GetItemSpell(item)

            if QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, false) == true then
              if CurrentButton <= QuestClicksMax then
                QuestClicksCreateButton(itemLink, itemName, itemTexture, CurrentButton)
                CurrentButton = CurrentButton + 1
              end
            end
		  end
        end
      end
    end
    
    -- Look for items in inventory slots
    for invslot = 1,19 do
      local item = GetInventoryItemLink("player", GetInventorySlotInfo(QuestClicksInventory[invslot]))
      if item then
        local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item) 
        local itemSpell, itemSpellRank = GetItemSpell(item)

        if QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, false) == true then
          if CurrentButton <= QuestClicksMax then
            QuestClicksCreateButton(itemLink, itemName, itemTexture, CurrentButton)
            CurrentButton = CurrentButton + 1
          end
        end
      end
	end

    -- Skin the buttons if ButtonFacade is enabled    
    if LBF then
      if QCBF_Group then
        QCBF_Group:Skin(QuestClicksProfile.ButtonFacade.Skin, QuestClicksProfile.ButtonFacade.Gloss, QuestClicksProfile.ButtonFacade.Backdrop)
      end
    end

    -- Hide any buttons that were not set up
    QuestClicksHideButtons(CurrentButton)
    
end

-- Set the Cooldown Timer for each item in the button bar
function UpdateCooldownFrames()
    for i=1,QuestClicksMax do
	  if (QuestClicksLinks[i]) then
	    --QuestClicks_DisplayMessage(QuestClicksLinks[i])
		local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", QuestClicksLinks[i])
        local cdstart, cdleft = GetItemCooldown(itemId)
        QuestClicksSubframes[i].Cooldown:SetCooldown(cdstart, cdleft)
	  end
    end
end

-- Convert a pre V1.10 variable list to a V1.10 Profile
function QuestClicksConvertProfile()
  QCDebugMessage("Converting Profile...")
  QuestClicksProfile["ProfileConverted"] = true
  QuestClicksProfile["Enabled"] = QuestClicksEnabled
  QuestClicksProfile["Direction"] = QuestClicksDirection
  QuestClicksProfile["Anchor"] = QuestClicksAnchor
  QuestClicksProfile["X"] = QuestClicksX
  QuestClicksProfile["Y"] = QuestClicksY
  QuestClicksProfile["Excludes"] = QuestClicksExcludes
  QuestClicksProfile["Includes"] = QuestClicksIncludes
  QuestClicksProfile["ButtonSize"] = QuestClicksButtonSize
  QuestClicksProfile["Locked"] = QuestClicksLocked
  QuestClicksProfile["AnchorHidden"] = QuestClicksAnchorHidden
  QuestClicksProfile["Collapsibles"] = QuestClicksCollapsibles
  QuestClicksProfile["Categories"] = QuestClicksCategories
end

-- Called by ButtonFacade when the skin for our buttons changes.  This lets us save
-- the new values to the config table.
function QuestClicks_BFSkinCallback(addon, skin, gloss, backdrop, group)
  if skin then
    QuestClicksProfile.ButtonFacade.Skin = skin
    QuestClicksProfile.ButtonFacade.Gloss = gloss
    QuestClicksProfile.ButtonFacade.Backdrop = backdrop  
  end
end


-- Determine which Scrolling Combat Text addon(s) are available.
local function QCCheckScrollingAddons()

  UseMSBT = false;
  UseSCT = false;
  UsePARROT = false;

  if aaSCT then
    if MikSBT then
	  UseMSBT = true;
	end
    if SCT then
	  UseSCT = true;
	end
    if Parrot then
	  UsePARROT = true;
	end
  end
end

---
---
--- Addon Primary Event Handler and Dispatcher
---
---

-- QuestClicksEvent handles any incoming events that the addon has registered for.
function QuestClicks_OnEvent(self, event, ...)

  QuestClicksProfile.DebugMode=false

  -- If the event we are responding to is "VARIABLES_LOADED", we need to
  -- create/show our buttons and reposition them based on the stored
  -- QuestClicksDirection variable
    if event=="VARIABLES_LOADED" then
	  if (QCDoneOnce==false) then
	    QCDoneOnce=true
        QCCheckScrollingAddons()
        QuestClicksUpdateButtons()
        QuestClicksRepositionButtons()
	    if QuestClicksProfile["ProfileConverted"] == false then
	      QuestClicksConvertProfile()
	    end
	  
	    if (QuestClicksProfile.Enabled==false) then
  	    QuestClicksTitleFrame:Hide()
	    end
      
      -- The following statements set default values for profile variables created after the initial switch to a profile system
      -- in version 1.10.  If the values don't exist in your profile table (ie, they are "nil") then they will be set to these
      -- defaults.
	  
        if QuestClicksProfile.ButtonsPerLine == nil then 
	      QuestClicksProfile.ButtonsPerLine = QuestClicksMax 
  	    end
	    if QuestClicksProfile.RelativePoint == nil then 
	      QuestClicksProfile.RelativePoint = "CENTER" 
	    end
		if type(QuestClicksProfile.RelativePoint) == "table" then QuestClicksProfile.RelativePoint = "CENTER" end
        if QuestClicksProfile.Padding == nil then 
  	      QuestClicksProfile.Padding = 0 
	    end
        if QuestClicksProfile.Categories[L("Quest Starters")] == nil then 
	      QuestClicksProfile.Categories[L("Quest Starters")] = true 
	    end
        if QuestClicksProfile.DebugMode == nil then 
	      QuestClicksProfile.DebugMode = false 
        end
        if QuestClicksProfile.QuestStarterNotify == nil then   
	      QuestClicksProfile.QuestStarterNotify = true 
	    end
        if QuestClicksProfile.EnableKeybinding == nil then 
	      QuestClicksProfile.EnableKeybinding = false 
	    end
        if QuestClicksProfile.Bindkey == nil then 
	      QuestClicksProfile.Bindkey = "CTRL-Q" 
	    end

        QuestClicksShowAnchor()
	    QuestClicksTitleFrame:SetPoint(QuestClicksProfile["RelativePoint"], UIParent, QuestClicksProfile["Anchor"], QuestClicksProfile["X"], QuestClicksProfile["Y"]);
        if QuestClicksProfile.AnchorHidden == true then
          QuestClicksHideAnchor()
        end

	    -- Set up ButtonFacade support if the addon is loaded.
	    if LBF then
	      -- Get a pointer to ButtonFacade
  	      local BF = LibStub("AceAddon-3.0"):GetAddon("ButtonFacade")

	      -- Register "QuestClicks" as a ButtonFacade supporting addon
		  -- This *SHOULD* add it to ButtonFacade's config menu, but it doesn't
		  -- always appear to do so unless a second module is loaded...
	      QCBF = BF:NewModule("QuestClicks")
		
		  -- Create a ButtonFacade group for QuestClicks
	      QCBF_Group = LBF:Group("QuestClicks")
		
		  -- Set the skin mode for the ButtonFacade QuestClicks group
	      QCBF_Group:Skin(QuestClicksProfile.ButtonFacade.Skin, QuestClicksProfile.ButtonFacade.Gloss, QuestClicksProfile.ButtonFacade.Backdrop)
		
		  -- Register a callback.  This is called every time the skin for the button group changes
		  -- This allows us an opportunity to save the new values in our config table.
	      LBF:RegisterSkinCallback("QuestClicks", QuestClicks_BFSkinCallback, "QuestClicks")
		
		  QuestClicks_DisplayMessage("Button Facade Support Enabled")
		
		  -- Add all of the buttons to the button group.
		  -- Note that the "buttondata" structure shouldn't be necessary, since the values are the defaults,
		  -- but it doesn't work properly without them.  *shrug*
	      for i=1, QuestClicksMax do	    
  		    local buttondata = {}
 		    buttondata["Icon"] = QuestClicksSubframes[i].Icon
		    buttondata["Cooldown"] = QuestClicksSubframes[i].Cooldown
		    QCBF_Group:AddButton(QuestClicksSubframes[i], buttondata)
		  end
	    end
	  end
    end
    
    --if event=="UPDATE_BINDINGS" then
    --  QCBindingKey = GetBindingKey("QUESTCLICKS_REPEAT");
    --  if (QuestClicksLastButton > 0 ) and (QCBindingKey ~= "") then
    --    SetBindingItem(QCBindingKey, QuestClicksLinks[QuestClicksLastButton]);
    --  end
    --end
    
	if event=="PLAYER_LEAVING_WORLD" then
	  --QuestClicks_DisplayMessage("PLAYER_LEAVING_WORLD")
	  QuestClicksStarted = false
	  QCSeenActivity = false
	end
	
    -- Essentially we are keeping the "BAG_UPDATE" event from processing
    -- our button list until the player is in the game.  This isn't a huge deal, but it
    -- saves a few cycles of processing the inventory for buttons as the game loads.
    if event=="PLAYER_ENTERING_WORLD" then
	  if (TKheading==nil) then
	    TKheading, TKcheckbox, TKbutton = LibStub("tekKonfig-Heading").new, LibStub("tekKonfig-Checkbox").new, LibStub("tekKonfig-Button").new
        TKslider = LibStub("tekKonfig-Slider").new
      end
      QuestClicksStarted = true
      QuestClicksUpdateButtons()
      QuestClicksRepositionButtons()
      QCDebugMessage("QuestClicks: PLAYER_ENTERING_WORLD")
    end
    
	if event=="PLAYER_CAMPING" then
	  QuestClicksStarted = false
	  QCSeenActivity = false
	end
	
    if QuestClicksStarted==true then 
	  local UpdateButtonsNow = false
	  
	  if event=="QUEST_FINISHED" or event == "PLAYER_REGEN_ENABLED" or event=="BANKFRAME_CLOSED" then
	    QCSeenActivity = true
        UpdateButtonsNow = true
      end
	  
      if (event=="BAG_UPDATE") and (QCSeenActivity==true) then
        UpdateButtonsNow = true
      end
        
	  -- if event=="CHAT_MSG_LOOT" then
	    -- if (arg1) then
		  -- if (arg1:find("You receive")) or (arg1:find("You create")) then
	        -- for itemLink in arg1:gmatch("|%x+|Hitem:.-|h.-|h|r") do
              -- local linkType, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", itemLink)
		  	  -- if QuestClicksEvaluateItem(itemLink, itemName, itemType, itemSpell, false) == true then
			    -- QuestClicks_DisplayMessage("Item " .. itemId .. " is a QuestClicks Item!")
				-- QCNeedLazyUpdate = true
			  -- end
            -- end
		  -- end
		-- end
	  -- end
	
	  if (event=="CHAT_MSG_SKILL") or (event=="PLAYER_LEVEL_UP") or (event=="TOYS_UPDATED") or (event=="COMPANION_LEARNED") then
  	    QC_BlockersCache = {}
		QC_RecipeClauseCache = {}
	  end
	
	-- If we are responding to BAG_UPDATE_COOLDOWN, update the cooldown frames
      if event=="BAG_UPDATE_COOLDOWN" or event == "ACTIONBAR_UPDATE_COOLDOWN" then
        QCDebugMessage("QuestClicks: " .. event)
        UpdateCooldownFrames()
		QCSeenActivity=true
		if (QCNeedLazyUpdate==true) then
		  UpdateButtonsNow = true
        end
      end

	  if (QuestClicksProfile.Enabled==false) then
	    UpdateButtonsNow=false
	  end
	  
      if UpdateButtonsNow==true then	
        if not InCombatLockdown() then
          QuestClicksUpdateButtons()
          QuestClicksRepositionButtons()
          QuestClicksBagUpdateCount = 0
		end
	  end
	  
    end
end
